@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2006 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2006-2009 B.A.T.M.A.N. contributors:
+ *
  * Thomas Lopatic, Corinna 'Elektra' Aichele, Axel Neumann,
  * Felix Fietkau, Marek Lindner
  *
@@ -31,6 +32,7 @@
 #include "batman.h"
 #include "originator.h"
 #include "schedule.h"
+#include "hna.h"
 
 
 
@@ -83,10 +85,6 @@ char *policy_routing_script = NULL;
 int policy_routing_pipe = 0;
 pid_t policy_routing_script_pid;
 
-unsigned char *hna_buff = NULL;
-
-uint8_t num_hna = 0;
-
 uint8_t found_ifs = 0;
 uint8_t active_ifs = 0;
 int32_t receive_max_sock = 0;
@@ -100,16 +98,11 @@ struct hashtable_t *orig_hash;
 struct list_head_first forw_list;
 struct list_head_first gw_list;
 struct list_head_first if_list;
-struct list_head_first hna_list;
-struct list_head_first hna_del_list;
-struct list_head_first hna_chg_list;
 
 struct vis_if vis_if;
 struct unix_if unix_if;
 struct debug_clients debug_clients;
 
-pthread_mutex_t hna_chg_list_mutex;
-
 unsigned char *vis_packet = NULL;
 uint16_t vis_packet_size = 0;
 
@@ -122,7 +115,10 @@ uint8_t minimum_recv = TQ_LOCAL_BIDRECT_RECV_MINIMUM;
 uint8_t global_win_size = TQ_GLOBAL_WINDOW_SIZE;
 uint8_t local_win_size = TQ_LOCAL_WINDOW_SIZE;
 uint8_t num_words = (TQ_LOCAL_WINDOW_SIZE / WORD_BIT_SIZE);
-uint8_t aggregation_enabled = 0;
+uint8_t aggregation_enabled = 1;
+
+int nat_tool_avail = -1;
+int8_t disable_client_nat = 0;
 
 
 
@@ -144,6 +140,7 @@ void usage(void)
 	fprintf( stderr, "       -s visualization server\n" );
 	fprintf( stderr, "       -v print version\n" );
 	fprintf( stderr, "       --policy-routing-script\n" );
+	fprintf( stderr, "       --disable-client-nat\n" );
 }
 
 
@@ -195,7 +192,7 @@ void verbose_usage(void)
 	fprintf( stderr, "          default: none, allowed values: IP\n\n" );
 	fprintf( stderr, "       -v print version\n" );
 	fprintf( stderr, "       --policy-routing-script send all routing table changes to the script\n" );
-	fprintf( stderr, "       --aggregation enable packet aggregation (experimental feature)\n" );
+	fprintf(stderr, "       --disable-client-nat deactivates the 'set tunnel NAT rules' feature (useful for half tunneling)\n");
 }
 
 
@@ -217,35 +214,6 @@ int is_batman_if(char *dev, struct batman_if **batman_if)
 	return 0;
 }
 
-
-
-void add_del_hna(struct orig_node *orig_node, int8_t del)
-{
-	uint16_t hna_buff_count = 0;
-	uint32_t hna, netmask;
-
-	while ( ( hna_buff_count + 1 ) * 5 <= orig_node->hna_buff_len ) {
-
-		memcpy( &hna, ( uint32_t *)&orig_node->hna_buff[ hna_buff_count * 5 ], 4 );
-		netmask = ( uint32_t )orig_node->hna_buff[ ( hna_buff_count * 5 ) + 4 ];
-
-		if ( ( netmask > 0 ) && ( netmask < 33 ) )
-			add_del_route( hna, netmask, orig_node->router->addr, orig_node->router->if_incoming->addr.sin_addr.s_addr, orig_node->batman_if->if_index, orig_node->batman_if->dev, BATMAN_RT_TABLE_NETWORKS, 0, del );
-
-		hna_buff_count++;
-
-	}
-
-	if ( del ) {
-
-		debugFree( orig_node->hna_buff, 1101 );
-		orig_node->hna_buff_len = 0;
-
-	}
-}
-
-
-
 void choose_gw(void)
 {
 	struct list_head *pos;
@@ -258,7 +226,7 @@ void choose_gw(void)
 
 
 	current_time = get_time_msec();
-	if ((routing_class == 0) || ((int64_t)(get_time_msec64() - (originator_interval * local_win_size)) < 0)) {
+	if ((routing_class == 0) || ((routing_class < 4) && ((int64_t)(get_time_msec64() - (originator_interval * local_win_size)) < 0))) {
 
 		prof_stop( PROF_choose_gw );
 		return;
@@ -373,18 +341,18 @@ void choose_gw(void)
 	prof_stop( PROF_choose_gw );
 }
 
-
-
 void update_routes(struct orig_node *orig_node, struct neigh_node *neigh_node, unsigned char *hna_recv_buff, int16_t hna_buff_len)
 {
 	char orig_str[ADDR_STR_LEN], next_str[ADDR_STR_LEN];
-	prof_start( PROF_update_routes );
+	struct neigh_node *old_router;
 
+	prof_start(PROF_update_routes);
+	debug_output(4, "update_routes() \n");
 
-	debug_output( 4, "update_routes() \n" );
+	old_router = orig_node->router;
 
-
-	if ( ( orig_node != NULL ) && ( orig_node->router != neigh_node ) ) {
+	/* also handles orig_node->router == NULL and neigh_node == NULL */
+	if ((orig_node != NULL) && (orig_node->router != neigh_node)) {
 
 		if ( ( orig_node != NULL ) && ( neigh_node != NULL ) ) {
 			addr_to_string( orig_node->orig, orig_str, ADDR_STR_LEN );
@@ -392,78 +360,61 @@ void update_routes(struct orig_node *orig_node, struct neigh_node *neigh_node, u
 			debug_output( 4, "Route to %s via %s\n", orig_str, next_str );
 		}
 
-		/* route altered or deleted */
-		if ( ( ( orig_node->router != NULL ) && ( neigh_node != NULL ) ) || ( neigh_node == NULL ) ) {
+		/* adds duplicated code but makes it more readable */
 
-			if ( neigh_node == NULL ) {
-				debug_output( 4, "Deleting previous route\n" );
-			} else {
-				debug_output( 4, "Route changed\n" );
-			}
+		/* new route added */
+		if ((orig_node->router == NULL) && (neigh_node != NULL)) {
 
-			/* remove old announced network(s) */
-			if ( orig_node->hna_buff_len > 0 )
-				add_del_hna( orig_node, 1 );
+			debug_output(4, "Adding new route\n");
 
-			add_del_route( orig_node->orig, 32, orig_node->router->addr, 0, orig_node->batman_if->if_index, orig_node->batman_if->dev, BATMAN_RT_TABLE_HOSTS, 0, 1 );
-
-		}
-
-		/* route altered or new route added */
-		if ( ( ( orig_node->router != NULL ) && ( neigh_node != NULL ) ) || ( orig_node->router == NULL ) ) {
-
-			if ( orig_node->router == NULL ) {
-				debug_output( 4, "Adding new route\n" );
-			} else {
-				debug_output( 4, "Route changed\n" );
-			}
-
-			add_del_route( orig_node->orig, 32, neigh_node->addr, neigh_node->if_incoming->addr.sin_addr.s_addr, neigh_node->if_incoming->if_index, neigh_node->if_incoming->dev, BATMAN_RT_TABLE_HOSTS, 0, 0 );
+			add_del_route(orig_node->orig, 32, neigh_node->addr, neigh_node->if_incoming->addr.sin_addr.s_addr,
+					neigh_node->if_incoming->if_index, neigh_node->if_incoming->dev, BATMAN_RT_TABLE_HOSTS, ROUTE_TYPE_UNICAST, ROUTE_ADD);
 
 			orig_node->batman_if = neigh_node->if_incoming;
 			orig_node->router = neigh_node;
 
 			/* add new announced network(s) */
-			if ( ( hna_buff_len > 0 ) && ( hna_recv_buff != NULL ) ) {
+			hna_global_add(orig_node, hna_recv_buff, hna_buff_len);
 
-				orig_node->hna_buff = debugMalloc( hna_buff_len, 101 );
-				orig_node->hna_buff_len = hna_buff_len;
+		/* route deleted */
+		} else if ((orig_node->router != NULL) && (neigh_node == NULL)) {
 
-				memmove( orig_node->hna_buff, hna_recv_buff, hna_buff_len );
+			debug_output(4, "Deleting previous route\n");
 
-				add_del_hna( orig_node, 0 );
+			/* remove old announced network(s) */
+			hna_global_del(orig_node);
 
-			}
+			add_del_route(orig_node->orig, 32, orig_node->router->addr, 0, orig_node->batman_if->if_index,
+					orig_node->batman_if->dev, BATMAN_RT_TABLE_HOSTS, ROUTE_TYPE_UNICAST, ROUTE_DEL);
 
+		/* route changed */
+		} else {
+
+			debug_output(4, "Route changed\n");
+
+			/* add new route */
+			add_del_route(orig_node->orig, 32, neigh_node->addr, neigh_node->if_incoming->addr.sin_addr.s_addr,
+					neigh_node->if_incoming->if_index, neigh_node->if_incoming->dev, BATMAN_RT_TABLE_HOSTS, ROUTE_TYPE_UNICAST, ROUTE_ADD);
+
+			/* delete old route */
+			add_del_route(orig_node->orig, 32, orig_node->router->addr, 0, orig_node->batman_if->if_index,
+					orig_node->batman_if->dev, BATMAN_RT_TABLE_HOSTS, ROUTE_TYPE_UNICAST, ROUTE_DEL);
+
+
+			orig_node->batman_if = neigh_node->if_incoming;
+			orig_node->router = neigh_node;
+
+			/* update announced network(s) */
+			hna_global_update(orig_node, hna_recv_buff, hna_buff_len, old_router);
 		}
 
 		orig_node->router = neigh_node;
 
-	} else if ( orig_node != NULL ) {
-
-		/* may be just HNA changed */
-		if ( ( hna_buff_len != orig_node->hna_buff_len ) || ( ( hna_buff_len > 0 ) && ( orig_node->hna_buff_len > 0 ) && ( memcmp( orig_node->hna_buff, hna_recv_buff, hna_buff_len ) != 0 ) ) ) {
-
-			if ( orig_node->hna_buff_len > 0 )
-				add_del_hna( orig_node, 1 );
-
-			if ( ( hna_buff_len > 0 ) && ( hna_recv_buff != NULL ) ) {
-
-				orig_node->hna_buff = debugMalloc( hna_buff_len, 102 );
-				orig_node->hna_buff_len = hna_buff_len;
-
-				memcpy( orig_node->hna_buff, hna_recv_buff, hna_buff_len );
-
-				add_del_hna( orig_node, 0 );
-
-			}
-
-		}
-
+	} else if (orig_node != NULL) {
+		hna_global_update(orig_node, hna_recv_buff, hna_buff_len, old_router);
 	}
 
-	prof_stop( PROF_update_routes );
-
+	prof_stop(PROF_update_routes);
 }
 
 
@@ -588,44 +539,7 @@ unsigned char get_gw_class(int down, int up)
 
 
 
-int isDuplicate(struct orig_node *orig_node, uint16_t seqno)
-{
-	struct list_head *neigh_pos;
-	struct neigh_node *neigh_node;
-
-	prof_start( PROF_is_duplicate );
-
-	list_for_each( neigh_pos, &orig_node->neigh_list ) {
-
-		neigh_node = list_entry( neigh_pos, struct neigh_node, list );
-
-		if ( get_bit_status( neigh_node->real_bits, orig_node->last_real_seqno, seqno ) ) {
-
-			prof_stop( PROF_is_duplicate );
-			return 1;
-
-		}
-
-	}
-
-	prof_stop( PROF_is_duplicate );
-
-	return 0;
-}
-
-
-
-int isBntog(uint32_t neigh, struct orig_node *orig_tog_node)
-{
-	if ( ( orig_tog_node->router != NULL ) && ( orig_tog_node->router->addr == neigh ) )
-		return 1;
-
-	return 0;
-}
-
-
-
-int isBidirectionalNeigh(struct orig_node *orig_node, struct orig_node *orig_neigh_node, struct bat_packet *in, uint32_t recv_time, struct batman_if *if_incoming, uint8_t is_duplicate)
+static int isBidirectionalNeigh(struct orig_node *orig_node, struct orig_node *orig_neigh_node, struct bat_packet *in, uint32_t recv_time, struct batman_if *if_incoming)
 {
 
 	struct list_head *list_pos;
@@ -712,52 +626,47 @@ int isBidirectionalNeigh(struct orig_node *orig_node, struct orig_node *orig_nei
 	return 0;
 }
 
-
-
-void generate_vis_packet(void)
+static void generate_vis_packet(void)
 {
 	struct hash_it_t *hashit = NULL;
 	struct orig_node *orig_node;
 	struct vis_data *vis_data;
 	struct list_head *list_pos;
 	struct batman_if *batman_if;
-	struct hna_node *hna_node;
 
-
-	if ( vis_packet != NULL ) {
-
-		debugFree( vis_packet, 1102 );
+	if (vis_packet != NULL) {
+		debugFree(vis_packet, 1102);
 		vis_packet = NULL;
 		vis_packet_size = 0;
-
 	}
 
 	vis_packet_size = sizeof(struct vis_packet);
-	vis_packet = debugMalloc( vis_packet_size, 104 );
+	vis_packet = debugMalloc(vis_packet_size, 104);
 
-	memcpy( &((struct vis_packet *)vis_packet)->sender_ip, (unsigned char *)&(((struct batman_if *)if_list.next)->addr.sin_addr.s_addr), 4 );
+	memcpy(&((struct vis_packet *)vis_packet)->sender_ip, (unsigned char *)&(((struct batman_if *)if_list.next)->addr.sin_addr.s_addr), 4);
 
 	((struct vis_packet *)vis_packet)->version = VIS_COMPAT_VERSION;
 	((struct vis_packet *)vis_packet)->gw_class = gateway_class;
 	((struct vis_packet *)vis_packet)->tq_max = TQ_MAX_VALUE;
 
 	/* neighbor list */
-	while ( NULL != ( hashit = hash_iterate( orig_hash, hashit ) ) ) {
+	while (NULL != (hashit = hash_iterate(orig_hash, hashit))) {
 
 		orig_node = hashit->bucket->data;
 
 		/* we interested in 1 hop neighbours only */
-		if ( ( orig_node->router != NULL ) && ( orig_node->orig == orig_node->router->addr ) && ( orig_node->router->tq_avg > 0 ) ) {
+		if ((orig_node->router != NULL) && (orig_node->orig == orig_node->router->addr) &&
+		    (orig_node->router->tq_avg > 0)) {
 
 			vis_packet_size += sizeof(struct vis_data);
 
-			vis_packet = debugRealloc( vis_packet, vis_packet_size, 105 );
+			vis_packet = debugRealloc(vis_packet, vis_packet_size, 105);
 
 			vis_data = (struct vis_data *)(vis_packet + vis_packet_size - sizeof(struct vis_data));
 
-			memcpy( &vis_data->ip, (unsigned char *)&orig_node->orig, 4 );
+			memcpy(&vis_data->ip, (unsigned char *)&orig_node->orig, 4);
 
-			vis_data->data = htons(orig_node->router->tq_avg);
+			vis_data->data = orig_node->router->tq_avg;
 			vis_data->type = DATA_TYPE_NEIGH;
 
 		}
@@ -765,65 +674,37 @@ void generate_vis_packet(void)
 	}
 
 	/* secondary interfaces */
-	if ( found_ifs > 1 ) {
+	if (found_ifs > 1) {
+		list_for_each(list_pos, &if_list) {
+			batman_if = list_entry(list_pos, struct batman_if, list);
 
-		list_for_each( list_pos, &if_list ) {
-
-			batman_if = list_entry( list_pos, struct batman_if, list );
-
-			if ( ((struct vis_packet *)vis_packet)->sender_ip == batman_if->addr.sin_addr.s_addr )
+			if (((struct vis_packet *)vis_packet)->sender_ip == batman_if->addr.sin_addr.s_addr)
 				continue;
 
 			vis_packet_size += sizeof(struct vis_data);
 
-			vis_packet = debugRealloc( vis_packet, vis_packet_size, 106 );
+			vis_packet = debugRealloc(vis_packet, vis_packet_size, 106);
 
 			vis_data = (struct vis_data *)(vis_packet + vis_packet_size - sizeof(struct vis_data));
 
-			memcpy( &vis_data->ip, (unsigned char *)&batman_if->addr.sin_addr.s_addr, 4 );
+			memcpy(&vis_data->ip, (unsigned char *)&batman_if->addr.sin_addr.s_addr, 4);
 
 			vis_data->data = 0;
 			vis_data->type = DATA_TYPE_SEC_IF;
-
 		}
-
 	}
 
 	/* hna announcements */
-	if ( num_hna > 0 ) {
+	vis_packet = hna_local_update_vis_packet(vis_packet, &vis_packet_size);
 
-		list_for_each( list_pos, &hna_list ) {
-
-			hna_node = list_entry( list_pos, struct hna_node, list );
-
-			vis_packet_size += sizeof(struct vis_data);
-
-			vis_packet = debugRealloc( vis_packet, vis_packet_size, 107 );
-
-			vis_data = (struct vis_data *)(vis_packet + vis_packet_size - sizeof(struct vis_data));
-
-			memcpy( &vis_data->ip, (unsigned char *)&hna_node->addr, 4 );
-
-			vis_data->data = hna_node->netmask;
-			vis_data->type = DATA_TYPE_HNA;
-
-		}
-
-	}
-
-
-	if ( vis_packet_size == sizeof(struct vis_packet) ) {
-
-		debugFree( vis_packet, 1107 );
+	if (vis_packet_size == sizeof(struct vis_packet)) {
+		debugFree(vis_packet, 1107);
 		vis_packet = NULL;
 		vis_packet_size = 0;
-
 	}
 }
 
-
-
-void send_vis_packet(void)
+static void send_vis_packet(void)
 {
 	generate_vis_packet();
 
@@ -831,9 +712,7 @@ void send_vis_packet(void)
 		send_udp_packet(vis_packet, vis_packet_size, &vis_if.addr, vis_if.sock, NULL);
 }
 
-
-
-uint8_t count_real_packets(struct bat_packet *in, uint32_t neigh, struct batman_if *if_incoming)
+static uint8_t count_real_packets(struct bat_packet *in, uint32_t neigh, struct batman_if *if_incoming)
 {
 	struct list_head *list_pos;
 	struct orig_node *orig_node;
@@ -883,22 +762,19 @@ uint8_t count_real_packets(struct bat_packet *in, uint32_t neigh, struct batman_
 	return is_duplicate;
 }
 
-
-
 int8_t batman(void)
 {
-	struct list_head *list_pos, *list_pos_tmp, *hna_pos, *hna_pos_tmp, *forw_pos_tmp, *prev_list_head;
+	struct list_head *list_pos, *forw_pos_tmp;
 	struct orig_node *orig_neigh_node, *orig_node;
 	struct batman_if *batman_if, *if_incoming;
-	struct hna_node *hna_node, *hna_node_exist;
 	struct forw_node *forw_node;
 	struct bat_packet *bat_packet;
-	uint32_t neigh, hna, netmask, debug_timeout, vis_timeout, select_timeout, curr_time;
+	uint32_t neigh, debug_timeout, vis_timeout, select_timeout, curr_time;
 	unsigned char in[2001], *hna_recv_buff;
-	char orig_str[ADDR_STR_LEN], neigh_str[ADDR_STR_LEN], ifaddr_str[ADDR_STR_LEN], oldorig_str[ADDR_STR_LEN];
-	int16_t hna_buff_count, hna_buff_len, packet_len, curr_packet_len;
+	char orig_str[ADDR_STR_LEN], neigh_str[ADDR_STR_LEN], ifaddr_str[ADDR_STR_LEN], prev_sender_str[ADDR_STR_LEN];
+	int16_t hna_buff_len, packet_len, curr_packet_len;
 	uint8_t forward_old, if_rp_filter_all_old, if_rp_filter_default_old, if_send_redirects_all_old, if_send_redirects_default_old;
-	uint8_t is_my_addr, is_my_orig, is_my_oldorig, is_broadcast, is_duplicate, is_bidirectional, is_bntog, has_directlink_flag;
+	uint8_t is_my_addr, is_my_orig, is_my_oldorig, is_broadcast, is_duplicate, is_bidirectional, has_directlink_flag;
 	int8_t res;
 
 
@@ -908,46 +784,23 @@ int8_t batman(void)
 		return(-1);
 
 	/* for profiling the functions */
-	prof_init( PROF_choose_gw, "choose_gw" );
-	prof_init( PROF_update_routes, "update_routes" );
-	prof_init( PROF_update_gw_list, "update_gw_list" );
-	prof_init( PROF_is_duplicate, "isDuplicate" );
-	prof_init( PROF_get_orig_node, "get_orig_node" );
-	prof_init( PROF_update_originator, "update_orig" );
-	prof_init( PROF_purge_originator, "purge_orig" );
-	prof_init( PROF_schedule_forward_packet, "schedule_forward_packet" );
-	prof_init( PROF_send_outstanding_packets, "send_outstanding_packets" );
+	prof_init(PROF_choose_gw, "choose_gw");
+	prof_init(PROF_update_routes, "update_routes");
+	prof_init(PROF_update_gw_list, "update_gw_list");
+	prof_init(PROF_is_duplicate, "isDuplicate");
+	prof_init(PROF_get_orig_node, "get_orig_node");
+	prof_init(PROF_update_originator, "update_orig");
+	prof_init(PROF_purge_originator, "purge_orig");
+	prof_init(PROF_schedule_forward_packet, "schedule_forward_packet");
+	prof_init(PROF_send_outstanding_packets, "send_outstanding_packets");
 
-	if ( !( list_empty( &hna_list ) ) ) {
-
-		list_for_each( list_pos, &hna_list ) {
-
-			hna_node = list_entry( list_pos, struct hna_node, list );
-
-			hna_buff = debugRealloc( hna_buff, ( num_hna + 1 ) * 5 * sizeof( unsigned char ), 15 );
-
-			memmove( &hna_buff[ num_hna * 5 ], ( unsigned char *)&hna_node->addr, 4 );
-			hna_buff[ ( num_hna * 5 ) + 4 ] = ( unsigned char )hna_node->netmask;
-
-			num_hna++;
-
-			/* add throw routing entries for own hna */
-			add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_NETWORKS, 1, 0 );
-			add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_HOSTS, 1, 0 );
-			add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_UNREACH, 1, 0 );
-			add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_TUNNEL, 1, 0 );
-
-		}
-
-	}
-
-	list_for_each( list_pos, &if_list ) {
-		batman_if = list_entry( list_pos, struct batman_if, list );
+	list_for_each(list_pos, &if_list) {
+		batman_if = list_entry(list_pos, struct batman_if, list);
 
 		batman_if->out.version = COMPAT_VERSION;
 		batman_if->out.flags = 0x00;
-		batman_if->out.ttl = ( batman_if->if_num > 0 ? 2 : TTL );
-		batman_if->out.gwflags = ( batman_if->if_num > 0 ? 0 : gateway_class );
+		batman_if->out.ttl = (batman_if->if_num > 0 ? 2 : TTL);
+		batman_if->out.gwflags = (batman_if->if_num > 0 ? 0 : gateway_class);
 		batman_if->out.seqno = 1;
 		batman_if->out.gwport = htons(GW_PORT);
 		batman_if->out.tq = TQ_MAX_VALUE;
@@ -955,24 +808,24 @@ int8_t batman(void)
 		schedule_own_packet(batman_if);
 	}
 
-	if_rp_filter_all_old = get_rp_filter( "all" );
-	if_rp_filter_default_old = get_rp_filter( "default" );
+	if_rp_filter_all_old = get_rp_filter("all");
+	if_rp_filter_default_old = get_rp_filter("default");
 
-	if_send_redirects_all_old = get_send_redirects( "all" );
-	if_send_redirects_default_old = get_send_redirects( "default" );
+	if_send_redirects_all_old = get_send_redirects("all");
+	if_send_redirects_default_old = get_send_redirects("default");
 
-	set_rp_filter( 0, "all" );
-	set_rp_filter( 0, "default" );
+	set_rp_filter(0, "all");
+	set_rp_filter(0, "default");
 
-	set_send_redirects( 0, "all" );
-	set_send_redirects( 0, "default" );
+	set_send_redirects(0, "all");
+	set_send_redirects(0, "default");
 
 	forward_old = get_forwarding();
 	set_forwarding(1);
 
-	while ( !is_aborted() ) {
+	while (!is_aborted()) {
 
-		debug_output( 4, " \n \n" );
+		debug_output( 4, " \n" );
 
 		/* harden select_timeout against sudden time change (e.g. ntpdate) */
 		curr_time = get_time_msec();
@@ -981,223 +834,158 @@ int8_t batman(void)
 
 		res = receive_packet(in, sizeof(in), &packet_len, &neigh, select_timeout, &if_incoming);
 
-		if (res < 0)
-			return -1;
+		/* on receive error the interface is deactivated in receive_packet() */
+		if (res < 1)
+			goto send_packets;
 
-		if (res > 0) {
+		curr_time = get_time_msec();
+		curr_packet_len = 0;
+		bat_packet = (struct bat_packet *)in;
 
-			curr_time = get_time_msec();
-			curr_packet_len = 0;
-			bat_packet = (struct bat_packet *)in;
+		addr_to_string(neigh, neigh_str, sizeof(neigh_str));
+		addr_to_string(if_incoming->addr.sin_addr.s_addr, ifaddr_str, sizeof(ifaddr_str));
 
-			addr_to_string(neigh, neigh_str, sizeof(neigh_str));
-			addr_to_string(if_incoming->addr.sin_addr.s_addr, ifaddr_str, sizeof(ifaddr_str));
+		while ((curr_packet_len + (int)sizeof(struct bat_packet) <= packet_len) &&
+			(curr_packet_len + (int)sizeof(struct bat_packet) + bat_packet->hna_len * 5 <= packet_len) &&
+			(curr_packet_len + (int)sizeof(struct bat_packet) + bat_packet->hna_len * 5 <= MAX_AGGREGATION_BYTES)) {
 
-			while ((curr_packet_len + sizeof(struct bat_packet) <= packet_len) &&
-				(curr_packet_len + sizeof(struct bat_packet) + bat_packet->hna_len * 5 <= packet_len) &&
-				(curr_packet_len + sizeof(struct bat_packet) + bat_packet->hna_len * 5 <= MAX_AGGREGATION_BYTES)) {
+			bat_packet = (struct bat_packet *)(in + curr_packet_len);
+			curr_packet_len += sizeof(struct bat_packet) + bat_packet->hna_len * 5;
 
-				bat_packet = (struct bat_packet *)(in + curr_packet_len);
-				curr_packet_len += sizeof(struct bat_packet) + bat_packet->hna_len * 5;
+			/* network to host order for our 16bit seqno */
+			bat_packet->seqno = ntohs(bat_packet->seqno);
 
-				/* network to host order for our 16bit seqno */
-				bat_packet->seqno = ntohs(bat_packet->seqno);
+			addr_to_string(bat_packet->orig, orig_str, sizeof(orig_str));
+			addr_to_string(bat_packet->prev_sender, prev_sender_str, sizeof(prev_sender_str));
 
-				addr_to_string(bat_packet->orig, orig_str, sizeof(orig_str));
-				addr_to_string(bat_packet->old_orig, oldorig_str, sizeof(oldorig_str));
+			is_my_addr = is_my_orig = is_my_oldorig = is_broadcast = 0;
 
-				is_my_addr = is_my_orig = is_my_oldorig = is_broadcast = is_duplicate = is_bidirectional = is_bntog = 0;
+			has_directlink_flag = (bat_packet->flags & DIRECTLINK ? 1 : 0);
 
-				has_directlink_flag = (bat_packet->flags & DIRECTLINK ? 1 : 0);
+			debug_output(4, "Received BATMAN packet via NB: %s, IF: %s %s (from OG: %s, via old OG: %s, seqno %d, tq %d, TTL %d, V %d, IDF %d) \n", neigh_str, if_incoming->dev, ifaddr_str, orig_str, prev_sender_str, bat_packet->seqno, bat_packet->tq, bat_packet->ttl, bat_packet->version, has_directlink_flag);
 
-				debug_output(4, "Received BATMAN packet via NB: %s, IF: %s %s (from OG: %s, via old OG: %s, seqno %d, tq %d, TTL %d, V %d, IDF %d) \n", neigh_str, if_incoming->dev, ifaddr_str, orig_str, oldorig_str, bat_packet->seqno, bat_packet->tq, bat_packet->ttl, bat_packet->version, has_directlink_flag);
+			hna_buff_len = bat_packet->hna_len * 5;
+			hna_recv_buff = (hna_buff_len > 4 ? (unsigned char *)(bat_packet + 1) : NULL);
 
-				hna_buff_len = bat_packet->hna_len * 5;
-				hna_recv_buff = (hna_buff_len > 4 ? (unsigned char *)(bat_packet + 1) : NULL);
+			list_for_each(list_pos, &if_list) {
 
-				list_for_each(list_pos, &if_list) {
+				batman_if = list_entry(list_pos, struct batman_if, list);
 
-					batman_if = list_entry(list_pos, struct batman_if, list);
+				if (neigh == batman_if->addr.sin_addr.s_addr)
+					is_my_addr = 1;
 
-					if (neigh == batman_if->addr.sin_addr.s_addr)
-						is_my_addr = 1;
+				if (bat_packet->orig == batman_if->addr.sin_addr.s_addr)
+					is_my_orig = 1;
 
-					if (bat_packet->orig == batman_if->addr.sin_addr.s_addr)
-						is_my_orig = 1;
+				if (neigh == batman_if->broad.sin_addr.s_addr)
+					is_broadcast = 1;
 
-					if (neigh == batman_if->broad.sin_addr.s_addr)
-						is_broadcast = 1;
-
-					if (bat_packet->old_orig == batman_if->addr.sin_addr.s_addr)
-						is_my_oldorig = 1;
-
-				}
-
-
-				if (bat_packet->gwflags != 0)
-					debug_output(4, "Is an internet gateway (class %i) \n", bat_packet->gwflags);
-
-				if (hna_recv_buff != NULL) {
-
-					debug_output(4, "HNA information received (%i HNA network%s): \n", hna_buff_len / 5, (hna_buff_len / 5 > 1 ? "s": ""));
-					hna_buff_count = 0;
-
-					while ((hna_buff_count + 1) * 5 <= hna_buff_len) {
-
-						memmove(&hna, (uint32_t *)&hna_recv_buff[hna_buff_count * 5], 4);
-						netmask = (uint32_t)hna_recv_buff[(hna_buff_count * 5) + 4];
-
-						addr_to_string(hna, orig_str, sizeof(orig_str));
-
-						if ((netmask > 0 ) && (netmask < 33))
-							debug_output(4, "hna: %s/%i\n", orig_str, netmask);
-						else
-							debug_output(4, "hna: %s/%i -> ignoring (invalid netmask) \n", orig_str, netmask);
-
-						hna_buff_count++;
-
-					}
-
-				}
-
-
-				if (bat_packet->version != COMPAT_VERSION) {
-
-					debug_output(4, "Drop packet: incompatible batman version (%i) \n", bat_packet->version);
-
-				} else if (is_my_addr) {
-
-					debug_output(4, "Drop packet: received my own broadcast (sender: %s) \n", neigh_str);
-
-				} else if (is_broadcast) {
-
-					debug_output(4, "Drop packet: ignoring all packets with broadcast source IP (sender: %s) \n", neigh_str);
-
-				} else if (is_my_orig) {
-
-					orig_neigh_node = get_orig_node(neigh);
-
-					/*debug_output( 4, "received my own OGM via NB lastTxIfSeqno: %d, currRxSeqno: %d, prevRxSeqno: %d, currRxSeqno-prevRxSeqno %d \n", ( if_incoming->out.seqno - 2 ), ((struct bat_packet *)&in)->seqno, orig_neigh_node->bidirect_link[if_incoming->if_num], ((struct bat_packet *)&in)->seqno - orig_neigh_node->bidirect_link[if_incoming->if_num] );*/
-
-					/* neighbour has to indicate direct link and it has to come via the corresponding interface */
-					/* if received seqno equals last send seqno save new seqno for bidirectional check */
-	// 				if ( ( ((struct bat_packet *)&in)->flags & DIRECTLINK ) && ( if_incoming->addr.sin_addr.s_addr == ((struct bat_packet *)&in)->orig ) && ( ((struct bat_packet *)&in)->seqno - if_incoming->out.seqno + 2 == 0 ) ) {
-	//
-	// 					orig_neigh_node->bidirect_link[if_incoming->if_num] = ((struct bat_packet *)&in)->seqno;
-	//
-	// 					debug_output( 4, "indicating bidirectional link - updating bidirect_link seqno \n" );
-	//
-	// 				} else {
-	//
-	// 					debug_output( 4, "NOT indicating bidirectional link - NOT updating bidirect_link seqno \n" );
-	//
-	// 				}
-
-					if ((has_directlink_flag) && (if_incoming->addr.sin_addr.s_addr == bat_packet->orig) && (bat_packet->seqno - if_incoming->out.seqno + 2 == 0)) {
-
-						debug_output(4, "count own bcast (is_my_orig): old = %i, ", orig_neigh_node->bcast_own_sum[if_incoming->if_num]);
-
-						bit_mark((TYPE_OF_WORD *)&(orig_neigh_node->bcast_own[if_incoming->if_num * num_words]), 0);
-						orig_neigh_node->bcast_own_sum[if_incoming->if_num] = bit_packet_count((TYPE_OF_WORD *)&(orig_neigh_node->bcast_own[if_incoming->if_num * num_words]));
-
-						debug_output(4, "new = %i \n", orig_neigh_node->bcast_own_sum[if_incoming->if_num]);
-
-					}
-
-					debug_output(4, "Drop packet: originator packet from myself (via neighbour) \n");
-
-				} else if (bat_packet->tq == 0) {
-
-					count_real_packets(bat_packet, neigh, if_incoming);
-
-					debug_output(4, "Drop packet: originator packet with tq is 0 \n");
-
-				} else if (is_my_oldorig) {
-
-					debug_output(4, "Drop packet: ignoring all rebroadcast echos (sender: %s) \n", neigh_str);
-
-				} else {
-
-					is_duplicate = count_real_packets(bat_packet, neigh, if_incoming);
-
-					orig_node = get_orig_node(bat_packet->orig);
-
-					/* if sender is a direct neighbor the sender ip equals originator ip */
-					orig_neigh_node = (bat_packet->orig == neigh ? orig_node : get_orig_node(neigh));
-
-					/* drop packet if sender is not a direct neighbor and if we no route towards it */
-					if ((bat_packet->orig != neigh) && (orig_neigh_node->router == NULL)) {
-
-						debug_output(4, "Drop packet: OGM via unkown neighbor! \n");
-
-					} else {
-
-						is_bidirectional = isBidirectionalNeigh(orig_node, orig_neigh_node, bat_packet, curr_time, if_incoming, is_duplicate);
-
-						/* update ranking if it is not a duplicate or has the same seqno and similar ttl as the non-duplicate */
-						if ((is_bidirectional) && ((!is_duplicate) || ((orig_node->last_real_seqno == bat_packet->seqno) && (orig_node->last_ttl - 3 <= bat_packet->ttl))))
-							update_orig(orig_node, bat_packet, neigh, if_incoming, hna_recv_buff, hna_buff_len, is_duplicate, curr_time);
-
-						/*is_bntog = isBntog( neigh, orig_node );*/
-
-						/* is single hop (direct) neighbour */
-						if (bat_packet->orig == neigh) {
-
-							/* it is our best route towards him */
-							/*if ( is_bidirectional && is_bntog ) {*/
-
-								/* mark direct link on incoming interface */
-								schedule_forward_packet(orig_node, bat_packet, neigh, 1, hna_recv_buff, hna_buff_len, if_incoming, curr_time);
-
-								debug_output(4, "Forward packet: rebroadcast neighbour packet with direct link flag \n");
-
-							/* if an unidirectional neighbour sends us a packet - retransmit it with unidirectional flag to tell him that we get its packets */
-							/* if a bidirectional neighbour sends us a packet - retransmit it with unidirectional flag if it is not our best link to it in order to prevent routing problems */
-								/*} else if ( ( is_bidirectional && !is_bntog ) || ( !is_bidirectional ) ) {
-
-								schedule_forward_packet( orig_node, (struct bat_packet *)in, 1, 1, hna_recv_buff, hna_buff_len, if_incoming );
-
-								debug_output( 4, "Forward packet: rebroadcast neighbour packet with direct link and unidirectional flag \n" );
-
-								}*/
-
-						/* multihop originator */
-						} else {
-
-							if (is_bidirectional) {
-
-								if (!is_duplicate) {
-
-									schedule_forward_packet(orig_node, bat_packet, neigh, 0, hna_recv_buff, hna_buff_len, if_incoming, curr_time);
-
-									debug_output(4, "Forward packet: rebroadcast originator packet \n");
-
-								} else {
-
-									debug_output(4, "Drop packet: duplicate packet received\n");
-
-								}
-
-							} else {
-
-								debug_output(4, "Drop packet: not received via bidirectional link\n");
-
-							}
-
-						}
-
-					}
-
-				}
+				if (bat_packet->prev_sender == batman_if->addr.sin_addr.s_addr)
+					is_my_oldorig = 1;
 
 			}
 
+
+			if (bat_packet->gwflags != 0)
+				debug_output(4, "Is an internet gateway (class %i) \n", bat_packet->gwflags);
+
+			if (bat_packet->version != COMPAT_VERSION) {
+				debug_output(4, "Drop packet: incompatible batman version (%i) \n", bat_packet->version);
+				goto send_packets;
+			}
+
+			if (is_my_addr) {
+				debug_output(4, "Drop packet: received my own broadcast (sender: %s) \n", neigh_str);
+				goto send_packets;
+			}
+
+			if (is_broadcast) {
+				debug_output(4, "Drop packet: ignoring all packets with broadcast source IP (sender: %s) \n", neigh_str);
+				goto send_packets;
+			}
+
+			if (is_my_orig) {
+				orig_neigh_node = get_orig_node(neigh);
+
+				if ((has_directlink_flag) && (if_incoming->addr.sin_addr.s_addr == bat_packet->orig) && (bat_packet->seqno - if_incoming->out.seqno + 2 == 0)) {
+
+					debug_output(4, "count own bcast (is_my_orig): old = %i, ", orig_neigh_node->bcast_own_sum[if_incoming->if_num]);
+
+					bit_mark((TYPE_OF_WORD *)&(orig_neigh_node->bcast_own[if_incoming->if_num * num_words]), 0);
+					orig_neigh_node->bcast_own_sum[if_incoming->if_num] = bit_packet_count((TYPE_OF_WORD *)&(orig_neigh_node->bcast_own[if_incoming->if_num * num_words]));
+
+					debug_output(4, "new = %i \n", orig_neigh_node->bcast_own_sum[if_incoming->if_num]);
+
+				}
+
+				debug_output(4, "Drop packet: originator packet from myself (via neighbour) \n");
+				goto send_packets;
+			}
+
+			if (bat_packet->tq == 0) {
+				count_real_packets(bat_packet, neigh, if_incoming);
+
+				debug_output(4, "Drop packet: originator packet with tq is 0 \n");
+				goto send_packets;
+			}
+
+			if (is_my_oldorig) {
+				debug_output(4, "Drop packet: ignoring all rebroadcast echos (sender: %s) \n", neigh_str);
+				goto send_packets;
+			}
+
+			is_duplicate = count_real_packets(bat_packet, neigh, if_incoming);
+
+			orig_node = get_orig_node(bat_packet->orig);
+
+			/* if sender is a direct neighbor the sender ip equals originator ip */
+			orig_neigh_node = (bat_packet->orig == neigh ? orig_node : get_orig_node(neigh));
+
+			/* drop packet if sender is not a direct neighbor and if we no route towards it */
+			if ((bat_packet->orig != neigh) && (orig_neigh_node->router == NULL)) {
+				debug_output(4, "Drop packet: OGM via unknown neighbor! \n");
+				goto send_packets;
+			}
+
+			is_bidirectional = isBidirectionalNeigh(orig_node, orig_neigh_node, bat_packet, curr_time, if_incoming);
+
+			/* update ranking if it is not a duplicate or has the same seqno and similar ttl as the non-duplicate */
+			if ((is_bidirectional) && ((!is_duplicate) ||
+			     ((orig_node->last_real_seqno == bat_packet->seqno) &&
+			     (orig_node->last_ttl - 3 <= bat_packet->ttl))))
+				update_orig(orig_node, bat_packet, neigh, if_incoming, hna_recv_buff, hna_buff_len, is_duplicate, curr_time);
+
+			/* is single hop (direct) neighbour */
+			if (bat_packet->orig == neigh) {
+
+				/* mark direct link on incoming interface */
+				schedule_forward_packet(orig_node, bat_packet, neigh, 1, hna_buff_len, if_incoming, curr_time);
+
+				debug_output(4, "Forward packet: rebroadcast neighbour packet with direct link flag \n");
+				goto send_packets;
+			}
+
+			/* multihop originator */
+			if (!is_bidirectional) {
+				debug_output(4, "Drop packet: not received via bidirectional link\n");
+				goto send_packets;
+			}
+
+			if (is_duplicate) {
+				debug_output(4, "Drop packet: duplicate packet received\n");
+				goto send_packets;
+			}
+
+			debug_output(4, "Forward packet: rebroadcast originator packet \n");
+
+			schedule_forward_packet(orig_node, bat_packet, neigh, 0, hna_buff_len, if_incoming, curr_time);
+
 		}
 
 
+send_packets:
 		send_outstanding_packets(curr_time);
 
-
 		if ((int)(curr_time - (debug_timeout + 1000)) > 0) {
 
 			debug_timeout = curr_time;
@@ -1225,153 +1013,30 @@ int8_t batman(void)
 
 			}
 
-			if (pthread_mutex_trylock(&hna_chg_list_mutex) == 0) {
-
-				if (!(list_empty(&hna_chg_list))) {
-
-					list_for_each_safe(list_pos, list_pos_tmp, &hna_chg_list) {
-
-						hna_node = list_entry(list_pos, struct hna_node, list);
-						addr_to_string( hna_node->addr, oldorig_str, sizeof(oldorig_str) );
-
-						hna_node_exist = NULL;
-						prev_list_head = (struct list_head *)&hna_list;
-
-						list_for_each_safe(hna_pos, hna_pos_tmp, &hna_list) {
-
-							hna_node_exist = list_entry(hna_pos, struct hna_node, list);
-
-							if ((hna_node->addr == hna_node_exist->addr) && (hna_node->netmask == hna_node_exist->netmask)) {
-
-								if (hna_node->del) {
-									debug_output(3, "Deleting HNA from announce network list: %s/%i\n", oldorig_str, hna_node->netmask );
-
-									add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_NETWORKS, 1, 1 );
-									add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_HOSTS, 1, 1 );
-									add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_UNREACH, 1, 1 );
-									add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_TUNNEL, 1, 1 );
-
-									list_del(prev_list_head, hna_pos, &hna_list);
-
-									debugFree(hna_node_exist, 1109);
-								} else {
-									debug_output(3, "Can't add HNA - already announcing network: %s/%i\n", oldorig_str, hna_node->netmask );
-								}
-
-								break;
-
-							}
-
-							prev_list_head = &hna_node_exist->list;
-							hna_node_exist = NULL;
-
-						}
-
-						if (hna_node_exist == NULL) {
-
-							if (hna_node->del) {
-								debug_output(3, "Can't delete HNA - network is not announced: %s/%i\n", oldorig_str, hna_node->netmask );
-							} else {
-								debug_output(3, "Adding HNA to announce network list: %s/%i\n", oldorig_str, hna_node->netmask );
-
-								/* add throw routing entries for own hna */
-								add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_NETWORKS, 1, 0 );
-								add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_HOSTS, 1, 0 );
-								add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_UNREACH, 1, 0 );
-								add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_TUNNEL, 1, 0 );
-
-								/* add node */
-								hna_node_exist = debugMalloc( sizeof(struct hna_node), 105 );
-								memset(hna_node_exist, 0, sizeof(struct hna_node));
-								INIT_LIST_HEAD(&hna_node_exist->list);
-
-								hna_node_exist->addr = hna_node->addr;
-								hna_node_exist->netmask = hna_node->netmask;
-
-								list_add_tail(&hna_node_exist->list, &hna_list);
-							}
-
-						}
-
-						list_del((struct list_head *)&hna_chg_list, list_pos, &hna_chg_list);
-						debugFree(hna_node, 1110);
-
-					}
-
-					if (hna_buff != NULL)
-						debugFree(hna_buff, 1111);
-
-					num_hna = 0;
-					hna_buff = NULL;
-
-					if (!(list_empty(&hna_list))) {
-
-						list_for_each(list_pos, &hna_list) {
-
-							hna_node = list_entry(list_pos, struct hna_node, list);
-
-							hna_buff = debugRealloc(hna_buff, ( num_hna + 1 ) * 5 * sizeof( unsigned char ), 16);
-
-							memmove(&hna_buff[ num_hna * 5 ], ( unsigned char *)&hna_node->addr, 4);
-							hna_buff[ ( num_hna * 5 ) + 4 ] = ( unsigned char )hna_node->netmask;
-
-							num_hna++;
-
-						}
-
-					}
-
-				}
-
-				if (pthread_mutex_unlock(&hna_chg_list_mutex) != 0)
-					debug_output(0, "Error - could not unlock mutex (hna_chg_list_mutex => 3): %s \n", strerror(errno));
-
-			}
-
+			hna_local_task_exec();
 		}
 
 	}
 
-
-	if ( debug_level > 0 )
-		printf( "Deleting all BATMAN routes\n" );
+	if (debug_level > 0)
+		printf("Deleting all BATMAN routes\n");
 
 	purge_orig(get_time_msec() + (5 * purge_timeout) + originator_interval);
 
-	hash_destroy( orig_hash );
+	hash_destroy(orig_hash);
 
+	list_for_each_safe(list_pos, forw_pos_tmp, &forw_list) {
 
-	list_for_each_safe( list_pos, hna_pos_tmp, &hna_list ) {
+		forw_node = list_entry(list_pos, struct forw_node, list);
 
-		hna_node = list_entry( list_pos, struct hna_node, list );
-
-		/* add throw routing entries for own hna */
-		add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_NETWORKS, 1, 1 );
-		add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_HOSTS, 1, 1 );
-		add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_UNREACH, 1, 1 );
-		add_del_route( hna_node->addr, hna_node->netmask, 0, 0, 0, "unknown", BATMAN_RT_TABLE_TUNNEL, 1, 1 );
-
-		debugFree( hna_node, 1103 );
+		list_del((struct list_head *)&forw_list, list_pos, &forw_list);
 
+		debugFree(forw_node->pack_buff, 1105);
+		debugFree(forw_node, 1106);
 	}
 
-	if ( hna_buff != NULL )
-		debugFree( hna_buff, 1104 );
-
-
-	list_for_each_safe( list_pos, forw_pos_tmp, &forw_list ) {
-
-		forw_node = list_entry( list_pos, struct forw_node, list );
-
-		list_del( (struct list_head *)&forw_list, list_pos, &forw_list );
-
-		debugFree( forw_node->pack_buff, 1105 );
-		debugFree( forw_node, 1106 );
-
-	}
-
-	if ( vis_packet != NULL )
-		debugFree( vis_packet, 1108 );
+	if (vis_packet != NULL)
+		debugFree(vis_packet, 1108);
 
 	set_forwarding( forward_old );
 
