@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2006 BATMAN contributors:
+ * Copyright (C) 2006-2009 B.A.T.M.A.N. contributors:
+ *
  * Thomas Lopatic, Corinna 'Elektra' Aichele, Axel Neumann, Marek Lindner, Andreas Langer
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -67,7 +68,7 @@ static struct hashtable_t *wip_hash;
 static struct hashtable_t *vip_hash;
 static struct list_head free_client_list;
 
-int init_module()
+int init_module(void)
 {
 
 	printk(KERN_DEBUG "B.A.T.M.A.N. gateway modul\n");
@@ -95,7 +96,7 @@ int init_module()
 	return(0);
 }
 
-void cleanup_module()
+void cleanup_module(void)
 {
 	struct gate_priv *priv;
 	struct free_client_data *entry, *next;
@@ -129,6 +130,7 @@ void cleanup_module()
 		if(entry->gw_client != NULL) {
 			gw_client = entry->gw_client;
 			list_del(&entry->list);
+			kfree(entry);
 			kfree(gw_client);
 		}
 
@@ -251,6 +253,7 @@ static int batgat_ioctl( struct inode *inode, struct file *file, unsigned int cm
 
 				gw_client = entry->gw_client;
 				list_del(&entry->list);
+				kfree(entry);
 				kfree(gw_client);
 
 			}
@@ -289,7 +292,7 @@ static int packet_recv_thread(void *data)
 
 	server_addr.sin_family = AF_INET;
 	server_addr.sin_addr.s_addr = INADDR_ANY;
-	server_addr.sin_port = htons( (unsigned short) BATMAN_PORT );
+	server_addr.sin_port = __constant_htons( (unsigned short) BATMAN_PORT );
 
 	if ( ( sock_create( PF_INET, SOCK_RAW, IPPROTO_RAW, &inet_sock ) ) < 0 ) {
 
@@ -362,7 +365,7 @@ static int packet_recv_thread(void *data)
 					if( ( jiffies - client_data->last_keep_alive ) / HZ > LEASE_TIME ) {
 
 						hash_remove_bucket(wip_hash, hashit);
-						hash_remove_bucket(vip_hash, hashit);
+						hash_remove(vip_hash, client_data);
 
 						tmp_entry = kmalloc(sizeof(struct free_client_data), GFP_KERNEL);
 						if(tmp_entry != NULL) {
@@ -383,7 +386,6 @@ static int packet_recv_thread(void *data)
 
 				if(client_data != NULL) {
 
-					buffer[0] = TUNNEL_DATA;
 					memcpy( &buffer[1], &client_data->vip_addr, sizeof( client_data->vip_addr ) );
 
 					iov.iov_base = buffer;
@@ -508,7 +510,7 @@ static int bat_netdev_xmit( struct sk_buff *skb, struct net_device *dev )
 
 		sa.sin_family = AF_INET;
 		sa.sin_addr.s_addr = client_data->wip_addr;
-		sa.sin_port = htons( (unsigned short)BATMAN_PORT );
+		sa.sin_port = __constant_htons( (unsigned short)BATMAN_PORT );
 
 		msg.msg_flags = MSG_NOSIGNAL | MSG_DONTWAIT;
 		msg.msg_name = &sa;
@@ -544,7 +546,7 @@ static int bat_netdev_close( struct net_device *dev )
 	return( 0 );
 }
 
-static int create_bat_netdev()
+static int create_bat_netdev(void)
 {
 
 	struct gate_priv *priv;
@@ -587,6 +589,7 @@ static struct gw_client *get_ip_addr(struct sockaddr_in *client_addr)
 
 	if (gw_client != NULL) {
 		printk(KERN_DEBUG "found client in hash");
+		spin_unlock(&hash_lock);
 		return gw_client;
 	}
 
@@ -594,6 +597,7 @@ static struct gw_client *get_ip_addr(struct sockaddr_in *client_addr)
 		printk(KERN_DEBUG "use free client from list");
 		gw_client = entry->gw_client;
 		list_del(&entry->list);
+		kfree(entry);
 		break;
 	}
 
@@ -690,12 +694,12 @@ static int kernel_recvmsg(struct socket *sock, struct msghdr *msg, struct iovec
 
 int compare_wip(void *data1, void *data2)
 {
-	return ( memcmp( data1, data2, 4 ) );
+	return ( !memcmp( data1, data2, 4 ) );
 }
 
 int compare_vip(void *data1, void *data2)
 {
-	return ( memcmp( data1 + 4, data2 + 4, 4 ) );
+	return ( !memcmp( data1 + 4, data2 + 4, 4 ) );
 }
 
 int choose_wip(void *data, int32_t size)
