@@ -1,4 +1,6 @@
-/* Copyright (C) 2006 B.A.T.M.A.N. contributors:
+/*
+ * Copyright (C) 2006-2009 B.A.T.M.A.N. contributors:
+ *
  * Simon Wunderlich, Marek Lindner
  *
  * This program is free software; you can redistribute it and/or
@@ -25,8 +27,7 @@
 #include "os.h"
 #include "batman.h"
 #include "originator.h"
-
-
+#include "hna.h"
 
 
 struct neigh_node * create_neighbor(struct orig_node *orig_node, struct orig_node *orig_neigh_node, uint32_t neigh, struct batman_if *if_incoming) {
@@ -57,10 +58,9 @@ struct neigh_node * create_neighbor(struct orig_node *orig_node, struct orig_nod
 
 /* needed for hash, compares 2 struct orig_node, but only their ip-addresses. assumes that
  * the ip address is the first field in the struct */
-int compare_orig( void *data1, void *data2 ) {
-
-	return ( memcmp( data1, data2, 4 ) );
-
+int compare_orig( void *data1, void *data2 )
+{
+	return (memcmp(data1, data2, 4) == 0 ? 1 : 0);
 }
 
 
@@ -206,8 +206,8 @@ void update_orig( struct orig_node *orig_node, struct bat_packet *in, uint32_t n
 	ring_buffer_set(neigh_node->tq_recv, &neigh_node->tq_index, in->tq);
 	neigh_node->tq_avg = ring_buffer_avg(neigh_node->tq_recv);
 
-// 	is_new_seqno = bit_get_packet( neigh_node->seq_bits, in->seqno - orig_node->last_seqno, 1 );
-// 	is_new_seqno = ! get_bit_status( neigh_node->real_bits, orig_node->last_real_seqno, in->seqno );
+/* 	is_new_seqno = bit_get_packet( neigh_node->seq_bits, in->seqno - orig_node->last_seqno, 1 );
+ 	is_new_seqno = ! get_bit_status( neigh_node->real_bits, orig_node->last_real_seqno, in->seqno ); */
 
 
 	if ( !is_duplicate ) {
@@ -219,8 +219,6 @@ void update_orig( struct orig_node *orig_node, struct bat_packet *in, uint32_t n
 
 	if ( ( neigh_node->tq_avg > max_tq ) || ( ( neigh_node->tq_avg == max_tq ) && ( neigh_node->orig_node->bcast_own_sum[if_incoming->if_num] > max_bcast_own ) ) || ( ( orig_node->router == neigh_node ) && ( neigh_node->tq_avg == max_tq ) ) ) {
 
-		max_tq = neigh_node->tq_avg;
-		max_bcast_own = neigh_node->orig_node->bcast_own_sum[if_incoming->if_num];
 		best_neigh_node = neigh_node;
 
 	}
@@ -232,7 +230,7 @@ void update_orig( struct orig_node *orig_node, struct bat_packet *in, uint32_t n
 		update_gw_list( orig_node, in->gwflags, in->gwport );
 
 	orig_node->gwflags = in->gwflags;
-
+	hna_global_check_tq(orig_node);
 
 	/* restart gateway selection if we have more packets and fast or late switching enabled */
 	if ((routing_class > 2) && (orig_node->gwflags != 0) && (curr_gateway != NULL)) {
@@ -358,16 +356,15 @@ void purge_orig(uint32_t curr_time)
 					addr_to_string( neigh_node->addr, neigh_str, ADDR_STR_LEN );
 					debug_output( 4, "Neighbour timeout: originator %s, neighbour: %s, last_valid %u \n", orig_str, neigh_str, neigh_node->last_valid );
 
-					if ( orig_node->router == neigh_node ) {
+					if (orig_node->router == neigh_node) {
 
 						/* we have to delete the route towards this node before it gets purged */
 						debug_output( 4, "Deleting previous route \n" );
 
 						/* remove old announced network(s) */
-						if ( orig_node->hna_buff_len > 0 )
-							add_del_hna( orig_node, 1 );
+						hna_global_del(orig_node);
 
-						add_del_route( orig_node->orig, 32, orig_node->router->addr, 0, orig_node->batman_if->if_index, orig_node->batman_if->dev, BATMAN_RT_TABLE_HOSTS, 0, 1 );
+						add_del_route(orig_node->orig, 32, orig_node->router->addr, 0, orig_node->batman_if->if_index, orig_node->batman_if->dev, BATMAN_RT_TABLE_HOSTS, ROUTE_TYPE_UNICAST, ROUTE_DEL);
 
 						/* if the neighbour is the route towards our gateway */
 						if ((curr_gateway != NULL) && (curr_gateway->orig_node == orig_node))
@@ -432,7 +429,7 @@ void purge_orig(uint32_t curr_time)
 
 
 
-void debug_orig() {
+void debug_orig(void) {
 
 	struct hash_it_t *hashit = NULL;
 	struct list_head *forw_pos, *orig_pos, *neigh_pos;
@@ -539,7 +536,7 @@ void debug_orig() {
 
 				debug_out_size = debug_out_size + snprintf( ( debug_out_str + debug_out_size ), ( sizeof(debug_out_str) - 1 - debug_out_size ), " %15s (%3i)", str, neigh_node->tq_avg);
 
-				if ( debug_out_size + 30 > sizeof(debug_out_str) - 1 ) {
+				if ( (unsigned int)(debug_out_size + 30) > sizeof(debug_out_str) - 1 ) {
 
 					debug_output( 1, "%s \n", debug_out_str );
 					debug_output( 4, "%s \n", debug_out_str );
@@ -550,8 +547,10 @@ void debug_orig() {
 
 			}
 
-			debug_output( 1, "%s \n", debug_out_str );
-			debug_output( 4, "%s \n", debug_out_str );
+			if (debug_out_size > 0) {
+				debug_output( 1, "%s \n", debug_out_str );
+				debug_output( 4, "%s \n", debug_out_str );
+			}
 
 		}
 
