@@ -1,6 +1,8 @@
 /*
- * Copyright (C) 2006 BATMAN contributors:
+ * Copyright (C) 2006-2009 BATMAN contributors:
+ *
  * Thomas Lopatic, Marek Lindner
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
  * License as published by the Free Software Foundation.
@@ -29,7 +31,6 @@
 #include <string.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
-#include <sys/stat.h>
 #include <net/if.h>
 #include <arpa/inet.h>
 #include <fcntl.h>
@@ -38,6 +39,7 @@
 
 #include "../os.h"
 #include "../batman.h"
+#include "../hna.h"
 
 #define IOCSETDEV 1
 
@@ -45,7 +47,7 @@ int8_t stop;
 
 
 
-int my_daemon() {
+static int my_daemon(void) {
 
 	int fd;
 
@@ -86,7 +88,7 @@ int my_daemon() {
 
 }
 
-void create_routing_pipe()
+static void create_routing_pipe(void)
 {
 	int fd[2], pipe_opts;
 
@@ -126,106 +128,22 @@ void create_routing_pipe()
 
 		}
 
-		if (execl(policy_routing_script, policy_routing_script, NULL) < 0)
+		signal(SIGINT, SIG_IGN);
+		signal(SIGTERM, SIG_IGN);
+		signal(SIGPIPE, SIG_IGN);
+
+		if (execl("/bin/sh", "/bin/sh", "-c", policy_routing_script, NULL) < 0)
 			printf("Could not execute '%s': %s\n", policy_routing_script, strerror(errno));
 
 	}
 }
 
-void add_hna_to_list(char *hna_string, int8_t del, uint8_t change)
-{
-	struct hna_node *hna_node;
-	struct in_addr tmp_ip_holder;
-	uint16_t netmask;
-	char *slash_ptr;
-
-	if ( ( slash_ptr = strchr( hna_string, '/' ) ) == NULL ) {
-
-		if (change) {
-			debug_output(3, "Invalid announced network (netmask is missing): %s\n", hna_string );
-			return;
-		} else {
-			printf( "Invalid announced network (netmask is missing): %s\n", hna_string );
-			exit(EXIT_FAILURE);
-		}
-
-	}
-
-	*slash_ptr = '\0';
-
-	if ( inet_pton( AF_INET, hna_string, &tmp_ip_holder ) < 1 ) {
-
-		*slash_ptr = '/';
-
-		if (change) {
-			debug_output(3, "Invalid announced network (IP is invalid): %s\n", hna_string );
-			return;
-		} else {
-			printf( "Invalid announced network (IP is invalid): %s\n", hna_string );
-			exit(EXIT_FAILURE);
-		}
-
-	}
-
-	errno = 0;
-
-	netmask = strtol( slash_ptr + 1, NULL, 10 );
-
-	if ( ( errno == ERANGE ) || ( errno != 0 && netmask == 0 ) ) {
-
-		if (change) {
-			return;
-		} else {
-			perror("strtol");
-			exit(EXIT_FAILURE);
-		}
-
-	}
-
-	if ( netmask < 1 || netmask > 32 ) {
-
-		*slash_ptr = '/';
-
-		if (change) {
-			debug_output(3, "Invalid announced network (netmask is invalid): %s\n", hna_string );
-			return;
-		} else {
-			printf( "Invalid announced network (netmask is invalid): %s\n", hna_string );
-			exit(EXIT_FAILURE);
-		}
-
-	}
-
-	tmp_ip_holder.s_addr = (tmp_ip_holder.s_addr & htonl(0xFFFFFFFF << (32 - netmask)));
-
-	hna_node = debugMalloc( sizeof(struct hna_node), 203 );
-	memset( hna_node, 0, sizeof(struct hna_node) );
-	INIT_LIST_HEAD( &hna_node->list );
-
-	hna_node->addr = tmp_ip_holder.s_addr;
-	hna_node->netmask = netmask;
-
-	if (change) {
-		hna_node->del = del;
-		list_add_tail( &hna_node->list, &hna_chg_list );
-	} else {
-		if (del)
-			list_add_tail( &hna_node->list, &hna_del_list );
-		else
-			list_add_tail( &hna_node->list, &hna_list );
-	}
-
-	*slash_ptr = '/';
-}
-
 void apply_init_args( int argc, char *argv[] ) {
 
 	struct in_addr tmp_ip_holder;
 	struct batman_if *batman_if;
-	struct hna_node *hna_node;
+	struct hna_task *hna_task;
 	struct debug_level_info *debug_level_info;
-	struct list_head *list_pos, *list_pos_tmp;
-	struct stat fstats;
 	uint8_t found_args = 1, batch_mode = 0, info_output = 0, was_hna = 0;
 	int8_t res;
 
@@ -239,7 +157,8 @@ void apply_init_args( int argc, char *argv[] ) {
 		{"policy-routing-script",     required_argument,       0, 'n'},
 		{"hop-penalty",     required_argument,       0, 'm'},
 		{"purge-timeout",     required_argument,       0, 'q'},
-		{"aggregation",     no_argument,       0, 'x'},
+		{"disable-aggregation",     no_argument,       0, 'x'},
+		{"disable-client-nat",     no_argument,       0, 'z'},
 		{0, 0, 0, 0}
 	};
 
@@ -247,22 +166,23 @@ void apply_init_args( int argc, char *argv[] ) {
 	stop = 0;
 	prog_name = argv[0];
 
+	if ( strstr( SOURCE_VERSION, "-" ) != NULL )
+		printf( "WARNING: You are using the unstable batman branch. If you are interested in *using* batman get the latest stable release !\n" );
+
 	while ( ( optchar = getopt_long( argc, argv, "a:A:bcd:hHio:g:p:r:s:vV", long_options, &option_index ) ) != -1 ) {
 
 		switch ( optchar ) {
 
 			case 'a':
-
-				add_hna_to_list(optarg, 0, 0);
+				hna_local_task_add_str(optarg, ROUTE_ADD, 0);
 
 				/* increment found_args only by one if optarg and optchar are directly following each other
 				   increment found_args by two if there is some space between the arguments */
-				found_args += ( ( *((char*)( optarg - 1)) == optchar ) ? 1 : 2 );
+				found_args += ((*((char*)(optarg - 1)) == optchar ) ? 1 : 2);
 				break;
 
 			case 'A':
-
-				add_hna_to_list(optarg, 1, 0);
+				hna_local_task_add_str(optarg, ROUTE_DEL, 0);
 
 				found_args += ((*((char*)( optarg - 1)) == optchar) ? 1 : 2);
 				break;
@@ -354,11 +274,6 @@ void apply_init_args( int argc, char *argv[] ) {
 			case 'n':
 				policy_routing_script = optarg;
 
-				if (lstat(policy_routing_script, &fstats) < 0) {
-					printf( "Could not get file information of '%s': %s\n", policy_routing_script, strerror(errno));
-					exit(EXIT_FAILURE);
-				}
-
 				found_args += ((*((char*)( optarg - 1)) == optchar) ? 1 : 2);
 				break;
 
@@ -457,7 +372,12 @@ void apply_init_args( int argc, char *argv[] ) {
 				exit(EXIT_SUCCESS);
 
 			case 'x':
-				aggregation_enabled++;
+				aggregation_enabled = 0;
+				found_args++;
+				break;
+
+			case 'z':
+				disable_client_nat = 1;
 				found_args++;
 				break;
 
@@ -480,14 +400,13 @@ void apply_init_args( int argc, char *argv[] ) {
 	if ( ( download_speed > 0 ) && ( upload_speed == 0 ) )
 		upload_speed = download_speed / 5;
 
-	if ( download_speed > 0 ) {
+	if (download_speed > 0) {
 
-		gateway_class = get_gw_class( download_speed, upload_speed );
-		get_gw_speeds( gateway_class, &download_speed, &upload_speed );
+		gateway_class = get_gw_class(download_speed, upload_speed);
+		get_gw_speeds(gateway_class, &download_speed, &upload_speed);
 
 	}
 
-
 	if ( ( gateway_class != 0 ) && ( routing_class != 0 ) ) {
 		fprintf( stderr, "Error - routing class can't be set while gateway class is in use !\n" );
 		usage();
@@ -507,30 +426,22 @@ void apply_init_args( int argc, char *argv[] ) {
 	if ( ( ( routing_class != 0 ) || ( gateway_class != 0 ) ) && ( !probe_tun(1) ) )
 		exit(EXIT_FAILURE);
 
-	if ( !unix_client ) {
+	if (!unix_client) {
 
-		if ( argc <= found_args ) {
-
-			fprintf( stderr, "Error - no interface specified\n" );
+		if (argc <= found_args) {
+			fprintf(stderr, "Error - no interface specified\n");
 			usage();
 			exit(EXIT_FAILURE);
-
 		}
 
+		if ((disable_client_nat) && (routing_class == 0))
+			fprintf(stderr, "Warning - the activated option '--disable-client-nat' has no effect without setting a routing class.\n");
+
+		nat_tool_avail = probe_nat_tool();
+
 		if (policy_routing_script != NULL)
 			create_routing_pipe();
 
-		/* delete ignored "HNAs to be removed" */
-		list_for_each_safe( list_pos, list_pos_tmp, &hna_del_list ) {
-
-			hna_node = list_entry( list_pos, struct hna_node, list );
-
-			list_del( (struct list_head *)&hna_del_list, list_pos, &hna_del_list );
-
-			debugFree( hna_node, 1299 );
-
-		}
-
 		signal( SIGINT, handler );
 		signal( SIGTERM, handler );
 		signal( SIGPIPE, SIG_IGN );
@@ -540,17 +451,16 @@ void apply_init_args( int argc, char *argv[] ) {
 		debug_clients.mutex = debugMalloc( sizeof(pthread_mutex_t *) * debug_level_max, 209 );
 		debug_clients.clients_num = debugMalloc( sizeof(int16_t) * debug_level_max, 209 );
 
-		for ( res = 0; res < debug_level_max; res++ ) {
+		for (res = 0; res < debug_level_max; res++) {
 
-			debug_clients.fd_list[res] = debugMalloc( sizeof(struct list_head_first), 204 );
+			debug_clients.fd_list[res] = debugMalloc(sizeof(struct list_head_first), 204);
 			((struct list_head_first *)debug_clients.fd_list[res])->next = debug_clients.fd_list[res];
 			((struct list_head_first *)debug_clients.fd_list[res])->prev = debug_clients.fd_list[res];
 
-			debug_clients.mutex[res] = debugMalloc( sizeof(pthread_mutex_t), 209 );
-			pthread_mutex_init( (pthread_mutex_t *)debug_clients.mutex[res], NULL );
+			debug_clients.mutex[res] = debugMalloc(sizeof(pthread_mutex_t), 209);
+			pthread_mutex_init((pthread_mutex_t *)debug_clients.mutex[res], NULL);
 
 			debug_clients.clients_num[res] = 0;
-
 		}
 
 		if ( flush_routes_rules(0) < 0 )
@@ -559,7 +469,7 @@ void apply_init_args( int argc, char *argv[] ) {
 		if ( flush_routes_rules(1) < 0 )
 			exit(EXIT_FAILURE);
 
-		while ( argc > found_args ) {
+		while (argc > found_args) {
 
 			if (argv[found_args][0] == '-') {
 				fprintf(stderr, "%s: invalid option -- %s\n", argv[0], argv[found_args]);
@@ -567,8 +477,8 @@ void apply_init_args( int argc, char *argv[] ) {
 				exit(EXIT_FAILURE);
 			}
 
-			batman_if = debugMalloc( sizeof(struct batman_if), 206 );
-			memset( batman_if, 0, sizeof(struct batman_if) );
+			batman_if = debugMalloc(sizeof(struct batman_if), 206);
+			memset(batman_if, 0, sizeof(struct batman_if));
 			INIT_LIST_HEAD( &batman_if->list );
 
 			batman_if->dev = argv[found_args];
@@ -577,22 +487,12 @@ void apply_init_args( int argc, char *argv[] ) {
 			batman_if->if_rp_filter_old = -1;
 			batman_if->if_send_redirects_old = -1;
 
-			list_add_tail( &batman_if->list, &if_list );
+			list_add_tail(&batman_if->list, &if_list);
 
 			init_interface(batman_if);
 
-			if (batman_if->if_num > 0) {
-
-				hna_node = debugMalloc( sizeof(struct hna_node), 207 );
-				memset( hna_node, 0, sizeof(struct hna_node) );
-				INIT_LIST_HEAD( &hna_node->list );
-
-				hna_node->addr = batman_if->addr.sin_addr.s_addr;
-				hna_node->netmask = 32;
-
-				list_add_tail( &hna_node->list, &hna_list );
-
-			}
+			if (batman_if->if_num > 0)
+				hna_local_task_add_ip(batman_if->addr.sin_addr.s_addr, 32, ROUTE_ADD);
 
 			if (batman_if->if_active) {
 
@@ -637,17 +537,17 @@ void apply_init_args( int argc, char *argv[] ) {
 		}
 
 		/* daemonize */
-		if ( debug_level == 0 ) {
+		if (debug_level == 0) {
 
-			if ( my_daemon() < 0 ) {
+			if (my_daemon() < 0) {
 
-				printf( "Error - can't fork to background: %s\n", strerror(errno) );
+				printf("Error - can't fork to background: %s\n", strerror(errno));
 				restore_defaults();
 				exit(EXIT_FAILURE);
 
 			}
 
-			openlog( "batmand", LOG_PID, LOG_DAEMON );
+			openlog("batmand", LOG_PID, LOG_DAEMON);
 
 		} else {
 
@@ -666,37 +566,30 @@ void apply_init_args( int argc, char *argv[] ) {
 		pthread_create( &unix_if.listen_thread_id, NULL, &unix_listen, NULL );
 
 		/* add rule for hna networks */
-		add_del_rule( 0, 0, BATMAN_RT_TABLE_NETWORKS, BATMAN_RT_PRIO_UNREACH - 1, 0, 1, 0 );
+		add_del_rule(0, 0, BATMAN_RT_TABLE_NETWORKS, BATMAN_RT_PRIO_UNREACH - 1, 0, RULE_TYPE_DST, RULE_ADD);
 
 		/* add unreachable routing table entry */
-		add_del_route( 0, 0, 0, 0, 0, "unknown", BATMAN_RT_TABLE_UNREACH, 2, 0 );
-
-		if ( routing_class > 0 ) {
-
-			if ( add_del_interface_rules( 0 ) < 0 ) {
+		add_del_route(0, 0, 0, 0, 0, "unknown", BATMAN_RT_TABLE_UNREACH, ROUTE_TYPE_UNREACHABLE, ROUTE_ADD);
 
+		if (routing_class > 0) {
+			if (add_del_interface_rules(RULE_ADD) < 0) {
 				restore_defaults();
 				exit(EXIT_FAILURE);
-
 			}
-
 		}
 
-		memset( &vis_if, 0, sizeof(vis_if) );
-
-		if ( vis_server ) {
+		memset(&vis_if, 0, sizeof(vis_if));
 
+		if (vis_server) {
 			vis_if.addr.sin_family = AF_INET;
 			vis_if.addr.sin_port = htons(PORT + 2);
 			vis_if.addr.sin_addr.s_addr = vis_server;
 			vis_if.sock = socket( PF_INET, SOCK_DGRAM, 0 );
-
 		}
 
-		if ( gateway_class != 0 )
+		if (gateway_class != 0)
 			init_interface_gw();
 
-
 		if ( debug_level > 0 ) {
 
 			printf( "debug level: %i\n", debug_level );
@@ -777,37 +670,29 @@ more_hna:
 		} else if (purge_timeout_opt) {
 
 			batch_mode = 1;
-			snprintf(unix_buff, 20, "q:%i", purge_timeout);
+			snprintf(unix_buff, 20, "q:%u", purge_timeout);
 
-		} else if ( info_output ) {
+		} else if (info_output) {
 
 			batch_mode = 1;
 			snprintf( unix_buff, 10, "i" );
 
-		} else if (!list_empty(&hna_list)) {
+		} else if (!list_empty(&hna_chg_list)) {
 
 			batch_mode = was_hna = 1;
-			hna_node = (struct hna_node *)hna_list.next;
-			addr_to_string( hna_node->addr, str1, sizeof(str1) );
-			snprintf( unix_buff, 30, "a:%s/%i", str1, hna_node->netmask );
+			hna_task = (struct hna_task *)hna_chg_list.next;
+			addr_to_string(hna_task->addr, str1, sizeof(str1));
+			snprintf(unix_buff, 30, "%c:%s/%i",
+			         (hna_task->route_action == ROUTE_ADD ? 'a' : 'A'),
+			         str1, hna_task->netmask);
 
-			list_del((struct list_head *)&hna_list, &hna_node->list, &hna_list);
-			debugFree(hna_node, 1298);
-
-		} else if (!list_empty( &hna_del_list)) {
-
-			batch_mode = was_hna = 1;
-			hna_node = (struct hna_node *)hna_del_list.next;
-			addr_to_string( hna_node->addr, str1, sizeof(str1) );
-			snprintf( unix_buff, 30, "A:%s/%i", str1, hna_node->netmask );
-
-			list_del((struct list_head *)&hna_del_list, &hna_node->list, &hna_del_list);
-			debugFree(hna_node, 1299);
+			list_del((struct list_head *)&hna_chg_list, &hna_task->list, &hna_chg_list);
+			debugFree(hna_task, 1298);
 
 		} else {
 
 			batch_mode = 1;
-			snprintf( unix_buff, 10, "y" );
+			snprintf(unix_buff, 10, "y");
 
 		}
 
@@ -873,7 +758,7 @@ close_con:
 
 		}
 
-		if ((was_hna) && ((!list_empty(&hna_list)) || (!list_empty(&hna_del_list))))
+		if ((was_hna) && (!list_empty(&hna_chg_list)))
 			goto more_hna;
 
 		exit(EXIT_SUCCESS);
@@ -882,7 +767,7 @@ close_con:
 
 }
 
-void interface_listen_sockets()
+void interface_listen_sockets(void)
 {
 	struct list_head *list_pos;
 	struct batman_if *batman_if;
@@ -902,7 +787,7 @@ void interface_listen_sockets()
 	}
 }
 
-int is_interface_up(char *dev)
+static int is_interface_up(char *dev)
 {
 	struct ifreq int_req;
 	int sock;
@@ -922,6 +807,7 @@ int is_interface_up(char *dev)
 	if (ioctl(sock, SIOCGIFADDR, &int_req) < 0)
 		goto failure;
 
+	close(sock);
 	return 1;
 
 failure:
@@ -942,8 +828,8 @@ void deactivate_interface(struct batman_if *batman_if)
 
 	if ((batman_if->netaddr > 0) && (batman_if->netmask > 0)) {
 
-		add_del_rule(batman_if->netaddr, batman_if->netmask, BATMAN_RT_TABLE_HOSTS, BATMAN_RT_PRIO_DEFAULT + batman_if->if_num, 0, 1, 1);
-		add_del_rule(batman_if->netaddr, batman_if->netmask, BATMAN_RT_TABLE_UNREACH, BATMAN_RT_PRIO_UNREACH + batman_if->if_num, 0, 1, 1);
+		add_del_rule(batman_if->netaddr, batman_if->netmask, BATMAN_RT_TABLE_HOSTS, BATMAN_RT_PRIO_DEFAULT + batman_if->if_num, 0, RULE_TYPE_DST, RULE_DEL);
+		add_del_rule(batman_if->netaddr, batman_if->netmask, BATMAN_RT_TABLE_UNREACH, BATMAN_RT_PRIO_UNREACH + batman_if->if_num, 0, RULE_TYPE_DST, RULE_DEL);
 
 	}
 
@@ -963,7 +849,7 @@ void deactivate_interface(struct batman_if *batman_if)
 	debug_output(3, "Interface deactivated: %s\n", batman_if->dev);
 }
 
-void activate_interface(struct batman_if *batman_if)
+static void activate_interface(struct batman_if *batman_if)
 {
 	struct ifreq int_req;
 	int on = 1, sock_opts;
@@ -1033,19 +919,20 @@ void activate_interface(struct batman_if *batman_if)
 #else
 	batman_if->if_index = 0;
 #endif
+	batman_if->wifi_if = is_wifi_interface(batman_if->dev, batman_if->udp_recv_sock);
 
-	if ( ioctl( batman_if->udp_recv_sock, SIOCGIFNETMASK, &int_req ) < 0 ) {
+	if (ioctl(batman_if->udp_recv_sock, SIOCGIFNETMASK, &int_req) < 0) {
 
-		debug_output(3, "Error - can't get netmask address of interface %s: %s\n", batman_if->dev, strerror(errno) );
+		debug_output(3, "Error - can't get netmask address of interface %s: %s\n", batman_if->dev, strerror(errno));
 		goto error;
 
 	}
 
-	batman_if->netaddr = ( ((struct sockaddr_in *)&int_req.ifr_addr)->sin_addr.s_addr & batman_if->addr.sin_addr.s_addr );
-	batman_if->netmask = bit_count( ((struct sockaddr_in *)&int_req.ifr_addr)->sin_addr.s_addr );
+	batman_if->netaddr = (((struct sockaddr_in *)&int_req.ifr_addr)->sin_addr.s_addr & batman_if->addr.sin_addr.s_addr);
+	batman_if->netmask = bit_count(((struct sockaddr_in *)&int_req.ifr_addr)->sin_addr.s_addr);
 
-	add_del_rule( batman_if->netaddr, batman_if->netmask, BATMAN_RT_TABLE_HOSTS, BATMAN_RT_PRIO_DEFAULT + batman_if->if_num, 0, 1, 0 );
-	add_del_rule( batman_if->netaddr, batman_if->netmask, BATMAN_RT_TABLE_UNREACH, BATMAN_RT_PRIO_UNREACH + batman_if->if_num, 0, 1, 0 );
+	add_del_rule(batman_if->netaddr, batman_if->netmask, BATMAN_RT_TABLE_HOSTS, BATMAN_RT_PRIO_DEFAULT + batman_if->if_num, 0, RULE_TYPE_DST, RULE_ADD);
+	add_del_rule(batman_if->netaddr, batman_if->netmask, BATMAN_RT_TABLE_UNREACH, BATMAN_RT_PRIO_UNREACH + batman_if->if_num, 0, RULE_TYPE_DST, RULE_ADD);
 
 	if ( ( batman_if->udp_send_sock = socket( PF_INET, SOCK_DGRAM, 0 ) ) < 0 ) {
 
@@ -1075,7 +962,7 @@ void activate_interface(struct batman_if *batman_if)
 
 	}
 
-	// make udp socket non blocking
+	/* make udp socket non blocking */
 	sock_opts = fcntl(batman_if->udp_send_sock, F_GETFL, 0);
 	fcntl(batman_if->udp_send_sock, F_SETFL, sock_opts | O_NONBLOCK);
 
@@ -1094,7 +981,7 @@ void activate_interface(struct batman_if *batman_if)
 	}
 
 	batman_if->out.orig = batman_if->addr.sin_addr.s_addr;
-	batman_if->out.old_orig = batman_if->addr.sin_addr.s_addr;
+	batman_if->out.prev_sender = batman_if->addr.sin_addr.s_addr;
 
 	batman_if->if_active = 1;
 	active_ifs++;
@@ -1126,11 +1013,12 @@ void init_interface(struct batman_if *batman_if)
 		activate_interface(batman_if);
 }
 
-void check_inactive_interfaces()
+void check_inactive_interfaces(void)
 {
 	struct list_head *list_pos;
 	struct batman_if *batman_if;
 
+	/* all available interfaces are active */
 	if (found_ifs == active_ifs)
 		return;
 
@@ -1142,10 +1030,27 @@ void check_inactive_interfaces()
 	}
 }
 
+void check_active_interfaces(void)
+{
+	struct list_head *list_pos;
+	struct batman_if *batman_if;
+
+	/* all available interfaces are deactive */
+	if (active_ifs == 0)
+		return;
+
+	list_for_each(list_pos, &if_list) {
+		batman_if = list_entry(list_pos, struct batman_if, list);
+
+		if ((batman_if->if_active) && (!is_interface_up(batman_if->dev)))
+			deactivate_interface(batman_if);
+	}
+}
 
 
 
-void init_interface_gw () {
+
+void init_interface_gw (void) {
 
 	int32_t sock_opts, err, skfd;
 	struct ifreq ifr;
@@ -1154,13 +1059,13 @@ void init_interface_gw () {
 	struct batman_if *batman_if = (struct batman_if *)if_list.next;
 
 
-	if ( ( batman_if->udp_tunnel_sock = use_gateway_module( batman_if->dev ) ) < 0 ) {
+	if ((batman_if->udp_tunnel_sock = use_gateway_module()) < 0) {
 
 		batman_if->addr.sin_port = htons(GW_PORT);
 
-		batman_if->udp_tunnel_sock = socket( PF_INET, SOCK_DGRAM, 0 );
+		batman_if->udp_tunnel_sock = socket(PF_INET, SOCK_DGRAM, 0);
 
-		if ( batman_if->udp_tunnel_sock < 0 ) {
+		if (batman_if->udp_tunnel_sock < 0) {
 
 			debug_output( 0, "Error - can't create tunnel socket: %s\n", strerror(errno) );
 			restore_defaults();
@@ -1186,66 +1091,63 @@ void init_interface_gw () {
 
 	} else {
 
-		ioc.universal = strlen( batman_if->dev );
+		ioc.universal = strlen(batman_if->dev);
 		ioc.exists = 0;
-		strncpy( ioc.dev_name, batman_if->dev, IFNAMSIZ - 1 );
+		strncpy(ioc.dev_name, batman_if->dev, IFNAMSIZ - 1);
 
-		if( ioctl( batman_if->udp_tunnel_sock, IOCSETDEV, &ioc ) < 0 ) {
-			debug_output( 0, "Error - can't add device %s: %s\n", batman_if->dev,strerror(errno) );
+		if (ioctl(batman_if->udp_tunnel_sock, IOCSETDEV, &ioc) < 0) {
+			debug_output(0, "Error - can't add device %s: %s\n", batman_if->dev,strerror(errno));
 			batman_if->dev = NULL;
 			restore_defaults();
 			exit(EXIT_FAILURE);
 		}
 
 		/* set ip address of gate device */
-
-		if( ( skfd = socket( AF_INET, SOCK_DGRAM, 0 ) ) < 0 ) {
+		if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
 			debug_output( 0, "Error - can't create socket to gate\n" );
 			restore_defaults();
 			exit( EXIT_FAILURE );
 		}
 
-
-		memset( &ifr, 0, sizeof( ifr ) );
+		memset(&ifr, 0, sizeof(ifr));
 		memset(&sin, 0, sizeof(struct sockaddr));
 
-		strncpy( ifr.ifr_name, ioc.dev_name, IFNAMSIZ - 1 );
+		strncpy(ifr.ifr_name, ioc.dev_name, IFNAMSIZ - 1);
 
 		sin.sin_family = AF_INET;
 		sin.sin_addr.s_addr = ioc.universal;
 		memcpy(&ifr.ifr_addr, &sin, sizeof(struct sockaddr));
 
-		if( ( err =  ioctl( skfd, SIOCSIFADDR, &ifr ) ) < 0 ) {
-			debug_output( 0, "Error - can't set IFADDR %s: %s\n", ioc.dev_name, strerror(err) );
-			close( skfd );
+		if ((err = ioctl(skfd, SIOCSIFADDR, &ifr)) < 0) {
+			debug_output(0, "Error - can't set IFADDR %s: %s\n", ioc.dev_name, strerror(err));
+			close(skfd);
 			restore_defaults();
 			exit( EXIT_FAILURE );
 		}
 
-
-		memset( &ifr, 0, sizeof( ifr ) );
-		strncpy( ifr.ifr_name, ioc.dev_name, IFNAMSIZ - 1 );
-		if( ioctl( skfd, SIOCGIFFLAGS, &ifr ) < 0) {
-			debug_output( 0, "Error - can't get IFFLAGS for %s: %s\n", ioc.dev_name, strerror(errno) );
-			close( skfd );
+		memset(&ifr, 0, sizeof(ifr));
+		strncpy(ifr.ifr_name, ioc.dev_name, IFNAMSIZ - 1);
+		if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) {
+			debug_output(0, "Error - can't get IFFLAGS for %s: %s\n", ioc.dev_name, strerror(errno));
+			close(skfd);
 			restore_defaults();
 			exit( EXIT_FAILURE );
 		}
 
-		strncpy( ifr.ifr_name, ioc.dev_name, IFNAMSIZ - 1 );
-		ifr.ifr_flags |= ( IFF_UP | IFF_RUNNING );
+		strncpy(ifr.ifr_name, ioc.dev_name, IFNAMSIZ - 1);
+		ifr.ifr_flags |= (IFF_UP | IFF_RUNNING);
 
-		if( ioctl( skfd, SIOCSIFFLAGS, &ifr ) < 0) {
-			debug_output( 0, "Error - can't set IFFLAGS for %s: %s\n", ioc.dev_name, strerror(errno) );
-			close( skfd );
+		if (ioctl(skfd, SIOCSIFFLAGS, &ifr) < 0) {
+			debug_output(0, "Error - can't set IFFLAGS for %s: %s\n", ioc.dev_name, strerror(errno));
+			close(skfd);
 			restore_defaults();
 			exit( EXIT_FAILURE );
 		}
 
-		close( skfd );
+		close(skfd);
 
-		if( !ioc.exists )
-			add_del_route( ioc.universal, 16, 0, 0, ioc.ifindex, ioc.dev_name, 254, 0, 0 );
+		if (!ioc.exists)
+			add_del_route(ioc.universal, 16, 0, 0, ioc.ifindex, ioc.dev_name, 254, ROUTE_TYPE_UNICAST, ROUTE_ADD);
 
 
 	}
