@@ -1,6 +1,8 @@
 /*
- * Copyright (C) 2006 BATMAN contributors:
+ * Copyright (C) 2006-2009 BATMAN contributors:
+ *
  * Marek Lindner
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
  * License as published by the Free Software Foundation.
@@ -35,10 +37,10 @@
 
 #include "../os.h"
 #include "../batman.h"
+#include "../hna.h"
 
 
-
-void debug_output( int8_t debug_prio, char *format, ... ) {
+void debug_output(int8_t debug_prio, char *format, ...) {
 
 	struct list_head *debug_pos;
 	struct debug_level_info *debug_level_info;
@@ -47,28 +49,28 @@ void debug_output( int8_t debug_prio, char *format, ... ) {
 
 
 	if (!log_facility_active) {
-		va_start( args, format );
-		vprintf( format, args );
-		va_end( args );
+		va_start(args, format);
+		vprintf(format, args);
+		va_end(args);
 		return;
 	}
 
-	if ( debug_prio == 0 ) {
+	if (debug_prio == 0) {
 
-		if ( debug_level == 0 ) {
+		if (debug_level == 0) {
 
-			va_start( args, format );
-			vsyslog( LOG_ERR, format, args );
-			va_end( args );
+			va_start(args, format);
+			vsyslog(LOG_ERR, format, args);
+			va_end(args);
 
-		} else if ( ( debug_level == 3 ) || ( debug_level == 4 ) ) {
+		} else if ((debug_level == 3) || (debug_level == 4)) {
 
-			if ( debug_level == 4 )
+			if (debug_level == 4)
 				printf("[%10u] ", get_time_msec());
 
-			va_start( args, format );
-			vprintf( format, args );
-			va_end( args );
+			va_start(args, format);
+			vprintf(format, args);
+			va_end(args);
 
 		}
 
@@ -80,46 +82,36 @@ void debug_output( int8_t debug_prio, char *format, ... ) {
 
 	}
 
+	if (debug_clients.clients_num[debug_prio_intern] < 1)
+		return;
 
-	if ( debug_clients.clients_num[debug_prio_intern] > 0 ) {
+	if (pthread_mutex_trylock((pthread_mutex_t *)debug_clients.mutex[debug_prio_intern] ) != 0) {
+		debug_output(0, "Warning - could not trylock mutex (debug_output): %s \n", strerror(EBUSY));
+		return;
+	}
 
-		if ( pthread_mutex_trylock( (pthread_mutex_t *)debug_clients.mutex[debug_prio_intern] ) == 0 ) {
+	va_start(args, format);
 
-			va_start( args, format );
+	list_for_each(debug_pos, (struct list_head *)debug_clients.fd_list[debug_prio_intern]) {
 
-			list_for_each( debug_pos, (struct list_head *)debug_clients.fd_list[debug_prio_intern] ) {
+		debug_level_info = list_entry(debug_pos, struct debug_level_info, list);
 
-				debug_level_info = list_entry(debug_pos, struct debug_level_info, list);
+		/* batman debug gets milliseconds prepended for better debugging */
+		if (debug_prio_intern == 3)
+			dprintf(debug_level_info->fd, "[%10u] ", get_time_msec());
 
-				if ( debug_prio_intern == 3 )
-					dprintf(debug_level_info->fd, "[%10u] ", get_time_msec());
+		if (((debug_level == 1) || (debug_level == 2)) && (debug_level_info->fd == 1) && (strncmp(format, "BOD", 3) == 0))
+			system("clear");
 
-				if ( ( ( debug_level == 1 ) || ( debug_level == 2 ) ) && ( debug_level_info->fd == 1 ) && ( strncmp( format, "BOD", 3 ) == 0 ) ) {
-
-					system( "clear" );
-
-				} else {
-
-					if ( ( ( debug_level != 1 ) && ( debug_level != 2 ) ) || ( debug_level_info->fd != 1 ) || ( strncmp( format, "EOD", 3 ) != 0 ) )
-						vdprintf( debug_level_info->fd, format, args );
-
-				}
-
-			}
-
-			va_end( args );
-
-			if ( pthread_mutex_unlock( (pthread_mutex_t *)debug_clients.mutex[debug_prio_intern] ) < 0 )
-				debug_output( 0, "Error - could not unlock mutex (debug_output): %s \n", strerror( errno ) );
-
-		} else {
-
-			debug_output( 0, "Warning - could not trylock mutex (debug_output): %s \n", strerror( EBUSY ) );
-
-		}
+		if (((debug_level != 1) && (debug_level != 2)) || (debug_level_info->fd != 1) || (strncmp(format, "EOD", 3) != 0))
+			vdprintf(debug_level_info->fd, format, args);
 
 	}
 
+	va_end(args);
+
+	if (pthread_mutex_unlock((pthread_mutex_t *)debug_clients.mutex[debug_prio_intern]) < 0)
+		debug_output(0, "Error - could not unlock mutex (debug_output): %s \n", strerror(errno));
 }
 
 
@@ -135,7 +127,7 @@ void internal_output(uint32_t sock)
 	dprintf(sock, "unix_socket_path=%s\n", UNIX_PATH);
 	dprintf(sock, "own_ogm_jitter=%i\n", JITTER);
 	dprintf(sock, "default_ttl=%i\n", TTL);
-	dprintf(sock, "originator_timeout=%i (default: %i)\n", purge_timeout, PURGE_TIMEOUT);
+	dprintf(sock, "originator_timeout=%u (default: %u)\n", purge_timeout, PURGE_TIMEOUT);
 	dprintf(sock, "tq_local_window_size=%i\n", TQ_LOCAL_WINDOW_SIZE);
 	dprintf(sock, "tq_global_window_size=%i\n", TQ_GLOBAL_WINDOW_SIZE);
 	dprintf(sock, "tq_local_bidirect_send_minimum=%i\n", TQ_LOCAL_BIDRECT_SEND_MINIMUM);
@@ -154,12 +146,12 @@ void internal_output(uint32_t sock)
 
 
 
-void *unix_listen( void *arg ) {
+void *unix_listen(void * BATMANUNUSED(arg)) {
 
 	struct unix_client *unix_client;
 	struct debug_level_info *debug_level_info;
 	struct list_head *list_pos, *unix_pos_tmp, *debug_pos, *debug_pos_tmp, *prev_list_head, *prev_list_head_unix;
-	struct hna_node *hna_node;
+	struct hna_local_entry *hna_local_entry;
 	struct batman_if *batman_if;
 	struct timeval tv;
 	struct sockaddr_un sun_addr;
@@ -178,13 +170,13 @@ void *unix_listen( void *arg ) {
 
 	max_sock = unix_if.unix_sock;
 
-	while ( !is_aborted() ) {
+	while (!is_aborted()) {
 
 		tv.tv_sec = 1;
 		tv.tv_usec = 0;
-		memcpy( &tmp_wait_sockets, &wait_sockets, sizeof(fd_set) );
+		memcpy(&tmp_wait_sockets, &wait_sockets, sizeof(fd_set));
 
-		res = select( max_sock + 1, &tmp_wait_sockets, NULL, NULL, &tv );
+		res = select(max_sock + 1, &tmp_wait_sockets, NULL, NULL, &tv);
 
 		if ( res > 0 ) {
 
@@ -235,36 +227,18 @@ void *unix_listen( void *arg ) {
 
 							/* debug_output( 3, "gateway: client sent data via unix socket: %s\n", buff ); */
 
-							if ( buff[0] == 'a' ) {
-
-								if ( status > 2 ) {
-
-									if ( pthread_mutex_lock(&hna_chg_list_mutex) != 0 )
-										debug_output(0, "Error - could not lock mutex (hna_chg_list_mutex => 1): %s \n", strerror(errno));
-
-									add_hna_to_list(buff + 2, 0, 1);
-
-									if ( pthread_mutex_unlock(&hna_chg_list_mutex) != 0 )
-										debug_output( 0, "Error - could not unlock mutex (hna_chg_list_mutex => 1): %s \n", strerror(errno) );
-
-									dprintf( unix_client->sock, "EOD\n" );
+							if (buff[0] == 'a') {
 
+								if (status > 2) {
+									hna_local_task_add_str(buff + 2, ROUTE_ADD, 1);
+									dprintf(unix_client->sock, "EOD\n");
 								}
 
-							} else if ( buff[0] == 'A' ) {
-
-								if ( status > 2 ) {
-
-									if ( pthread_mutex_lock(&hna_chg_list_mutex) != 0 )
-										debug_output(0, "Error - could not lock mutex (hna_chg_list_mutex => 2): %s \n", strerror(errno));
-
-									add_hna_to_list(buff + 2, 1, 1);
-
-									if ( pthread_mutex_unlock(&hna_chg_list_mutex) != 0 )
-										debug_output( 0, "Error - could not unlock mutex (hna_chg_list_mutex => 2): %s \n", strerror(errno) );
-
-									dprintf( unix_client->sock, "EOD\n" );
+							} else if (buff[0] == 'A') {
 
+								if (status > 2) {
+									hna_local_task_add_str(buff + 2, ROUTE_DEL, 1);
+									dprintf(unix_client->sock, "EOD\n");
 								}
 
 							} else if ( buff[0] == 'd' ) {
@@ -347,7 +321,7 @@ void *unix_listen( void *arg ) {
 											if ((routing_class != 0) && (curr_gateway != NULL))
 												del_default_route();
 
-											add_del_interface_rules(1);
+											add_del_interface_rules(RULE_DEL);
 											routing_class = 0;
 
 										}
@@ -402,9 +376,9 @@ void *unix_listen( void *arg ) {
 											}
 
 											if ((tmp_unix_value > 0) && (routing_class == 0))
-												add_del_interface_rules(0);
+												add_del_interface_rules(RULE_ADD);
 											else if ((tmp_unix_value == 0) && (routing_class > 0))
-												add_del_interface_rules(1);
+												add_del_interface_rules(RULE_DEL);
 
 											routing_class = tmp_unix_value;
 
@@ -437,37 +411,35 @@ void *unix_listen( void *arg ) {
 
 								dprintf( unix_client->sock, "EOD\n" );
 
-							} else if ( buff[0] == 'y' ) {
+							} else if (buff[0] == 'y') {
 
-								dprintf( unix_client->sock, "%s", prog_name );
+								dprintf(unix_client->sock, "%s", prog_name);
 
-								if ( routing_class > 0 )
-									dprintf( unix_client->sock, " -r %i", routing_class );
+								if (routing_class > 0)
+									dprintf(unix_client->sock, " -r %i", routing_class);
 
-								if ( pref_gateway > 0 ) {
+								if (pref_gateway > 0) {
 
 									addr_to_string( pref_gateway, str, sizeof (str) );
 
-									dprintf( unix_client->sock, " -p %s", str );
+									dprintf(unix_client->sock, " -p %s", str);
 
 								}
 
-								if ( gateway_class > 0 ) {
+								if (gateway_class > 0) {
 
-									get_gw_speeds( gateway_class, &download_speed, &upload_speed );
+									get_gw_speeds(gateway_class, &download_speed, &upload_speed);
 
-									dprintf( unix_client->sock, " -g %i%s/%i%s", ( download_speed > 2048 ? download_speed / 1024 : download_speed ), ( download_speed > 2048 ? "MBit" : "KBit" ), ( upload_speed > 2048 ? upload_speed / 1024 : upload_speed ), ( upload_speed > 2048 ? "MBit" : "KBit" ) );
+									dprintf(unix_client->sock, " -g %i%s/%i%s", (download_speed > 2048 ? download_speed / 1024 : download_speed), (download_speed > 2048 ? "MBit" : "KBit"), (upload_speed > 2048 ? upload_speed / 1024 : upload_speed), (upload_speed > 2048 ? "MBit" : "KBit"));
 
 								}
 
-								list_for_each( debug_pos, &hna_list ) {
+								list_for_each(debug_pos, &hna_list) {
 
-									hna_node = list_entry( debug_pos, struct hna_node, list );
-
-									addr_to_string( hna_node->addr, str, sizeof (str) );
-
-									dprintf( unix_client->sock, " -a %s/%i", str, hna_node->netmask );
+									hna_local_entry = list_entry(debug_pos, struct hna_local_entry, list);
 
+									addr_to_string(hna_local_entry->addr, str, sizeof (str));
+									dprintf(unix_client->sock, " -a %s/%i", str, hna_local_entry->netmask);
 								}
 
 								if (debug_level != 0)
@@ -484,18 +456,24 @@ void *unix_listen( void *arg ) {
 								if (policy_routing_script != NULL)
 									dprintf(unix_client->sock, " --policy-routing-script %s", policy_routing_script);
 
-								if (hop_penalty != 0)
+								if (hop_penalty != TQ_HOP_PENALTY)
 									dprintf(unix_client->sock, " --hop-penalty %i", hop_penalty);
 
-								list_for_each( debug_pos, &if_list ) {
+								if (!aggregation_enabled)
+									dprintf(unix_client->sock, " --disable-aggregation");
 
-									batman_if = list_entry( debug_pos, struct batman_if, list );
+								if (purge_timeout != PURGE_TIMEOUT)
+									dprintf(unix_client->sock, " --purge-timeout %u", purge_timeout);
 
-									dprintf( unix_client->sock, " %s", batman_if->dev );
+								list_for_each(debug_pos, &if_list) {
+
+									batman_if = list_entry(debug_pos, struct batman_if, list);
+
+									dprintf(unix_client->sock, " %s", batman_if->dev);
 
 								}
 
-								dprintf( unix_client->sock, "\nEOD\n" );
+								dprintf(unix_client->sock, "\nEOD\n");
 
 							}
 
@@ -558,9 +536,9 @@ void *unix_listen( void *arg ) {
 
 			}
 
-		} else if ( ( res < 0 ) && ( errno != EINTR ) ) {
+		} else if ((res < 0) && (errno != EINTR)) {
 
-			debug_output( 0, "Error - can't select: %s\n", strerror(errno) );
+			debug_output(0, "Error - can't select (unix_listen): %s\n", strerror(errno));
 			break;
 
 		}
