@@ -1,4 +1,6 @@
-/* Copyright (C) 2006 B.A.T.M.A.N. contributors:
+/* 
+ * Copyright (C) 2006-2009 B.A.T.M.A.N. contributors:
+ *
  * Simon Wunderlich, Marek Lindner
  *
  * This program is free software; you can redistribute it and/or
@@ -19,5 +21,5 @@
 
 
 void schedule_own_packet( struct batman_if *batman_if );
-void schedule_forward_packet(struct orig_node *orig_node, struct bat_packet *in, uint32_t neigh, uint8_t directlink, unsigned char *hna_recv_buff, int16_t hna_buff_len, struct batman_if *if_outgoing, uint32_t curr_time);
+void schedule_forward_packet(struct orig_node *orig_node, struct bat_packet *in, uint32_t neigh, uint8_t directlink, int16_t hna_buff_len, struct batman_if *if_outgoing, uint32_t curr_time);
 void send_outstanding_packets(uint32_t curr_time);
